# -*- ruby encoding: utf-8 -*-
#############################################################
# This file was automatically generated on 2021-05-06.      #
#                                                           #
# Ruby Bindings Version 2.1.29                              #
#                                                           #
# If you have a bugfix for this file and want to commit it, #
# please fix the bug in the generator. You can find a link  #
# to the generators git repository on tinkerforge.com       #
#############################################################

require_relative './ip_connection'

module Tinkerforge
  # Drives one brushed DC motor with up to 28V and 5A (peak)
  class BrickDC < Device
    DEVICE_IDENTIFIER = 11 # :nodoc:
    DEVICE_DISPLAY_NAME = 'DC Brick' # :nodoc:

    # This callback is triggered when the input voltage drops below the value set by
    # BrickDC#set_minimum_voltage. The parameter is the current voltage.
    CALLBACK_UNDER_VOLTAGE = 21

    # This callback is triggered if either the current consumption
    # is too high (above 5A) or the temperature of the driver chip is too high
    # (above 175°C). These two possibilities are essentially the same, since the
    # temperature will reach this threshold immediately if the motor consumes too
    # much current. In case of a voltage below 3.3V (external or stack) this
    # callback is triggered as well.
    # 
    # If this callback is triggered, the driver chip gets disabled at the same time.
    # That means, BrickDC#enable has to be called to drive the motor again.
    # 
    # .. note::
    #  This callback only works in Drive/Brake mode (see BrickDC#set_drive_mode). In
    #  Drive/Coast mode it is unfortunately impossible to reliably read the
    #  overcurrent/overtemperature signal from the driver chip.
    CALLBACK_EMERGENCY_SHUTDOWN = 22

    # This callback is triggered whenever a set velocity is reached. For example:
    # If a velocity of 0 is present, acceleration is set to 5000 and velocity
    # to 10000, the CALLBACK_VELOCITY_REACHED callback will be triggered after about
    # 2 seconds, when the set velocity is actually reached.
    # 
    # .. note::
    #  Since we can't get any feedback from the DC motor, this only works if the
    #  acceleration (see BrickDC#set_acceleration) is set smaller or equal to the
    #  maximum acceleration of the motor. Otherwise the motor will lag behind the
    #  control value and the callback will be triggered too early.
    CALLBACK_VELOCITY_REACHED = 23

    # This callback is triggered with the period that is set by
    # BrickDC#set_current_velocity_period. The parameter is the *current*
    # velocity used by the motor.
    # 
    # The CALLBACK_CURRENT_VELOCITY callback is only triggered after the set period
    # if there is a change in the velocity.
    CALLBACK_CURRENT_VELOCITY = 24

    FUNCTION_SET_VELOCITY = 1 # :nodoc:
    FUNCTION_GET_VELOCITY = 2 # :nodoc:
    FUNCTION_GET_CURRENT_VELOCITY = 3 # :nodoc:
    FUNCTION_SET_ACCELERATION = 4 # :nodoc:
    FUNCTION_GET_ACCELERATION = 5 # :nodoc:
    FUNCTION_SET_PWM_FREQUENCY = 6 # :nodoc:
    FUNCTION_GET_PWM_FREQUENCY = 7 # :nodoc:
    FUNCTION_FULL_BRAKE = 8 # :nodoc:
    FUNCTION_GET_STACK_INPUT_VOLTAGE = 9 # :nodoc:
    FUNCTION_GET_EXTERNAL_INPUT_VOLTAGE = 10 # :nodoc:
    FUNCTION_GET_CURRENT_CONSUMPTION = 11 # :nodoc:
    FUNCTION_ENABLE = 12 # :nodoc:
    FUNCTION_DISABLE = 13 # :nodoc:
    FUNCTION_IS_ENABLED = 14 # :nodoc:
    FUNCTION_SET_MINIMUM_VOLTAGE = 15 # :nodoc:
    FUNCTION_GET_MINIMUM_VOLTAGE = 16 # :nodoc:
    FUNCTION_SET_DRIVE_MODE = 17 # :nodoc:
    FUNCTION_GET_DRIVE_MODE = 18 # :nodoc:
    FUNCTION_SET_CURRENT_VELOCITY_PERIOD = 19 # :nodoc:
    FUNCTION_GET_CURRENT_VELOCITY_PERIOD = 20 # :nodoc:
    FUNCTION_SET_SPITFP_BAUDRATE_CONFIG = 231 # :nodoc:
    FUNCTION_GET_SPITFP_BAUDRATE_CONFIG = 232 # :nodoc:
    FUNCTION_GET_SEND_TIMEOUT_COUNT = 233 # :nodoc:
    FUNCTION_SET_SPITFP_BAUDRATE = 234 # :nodoc:
    FUNCTION_GET_SPITFP_BAUDRATE = 235 # :nodoc:
    FUNCTION_GET_SPITFP_ERROR_COUNT = 237 # :nodoc:
    FUNCTION_ENABLE_STATUS_LED = 238 # :nodoc:
    FUNCTION_DISABLE_STATUS_LED = 239 # :nodoc:
    FUNCTION_IS_STATUS_LED_ENABLED = 240 # :nodoc:
    FUNCTION_GET_PROTOCOL1_BRICKLET_NAME = 241 # :nodoc:
    FUNCTION_GET_CHIP_TEMPERATURE = 242 # :nodoc:
    FUNCTION_RESET = 243 # :nodoc:
    FUNCTION_WRITE_BRICKLET_PLUGIN = 246 # :nodoc:
    FUNCTION_READ_BRICKLET_PLUGIN = 247 # :nodoc:
    FUNCTION_GET_IDENTITY = 255 # :nodoc:

    DRIVE_MODE_DRIVE_BRAKE = 0 # :nodoc:
    DRIVE_MODE_DRIVE_COAST = 1 # :nodoc:
    COMMUNICATION_METHOD_NONE = 0 # :nodoc:
    COMMUNICATION_METHOD_USB = 1 # :nodoc:
    COMMUNICATION_METHOD_SPI_STACK = 2 # :nodoc:
    COMMUNICATION_METHOD_CHIBI = 3 # :nodoc:
    COMMUNICATION_METHOD_RS485 = 4 # :nodoc:
    COMMUNICATION_METHOD_WIFI = 5 # :nodoc:
    COMMUNICATION_METHOD_ETHERNET = 6 # :nodoc:
    COMMUNICATION_METHOD_WIFI_V2 = 7 # :nodoc:

    # Creates an object with the unique device ID <tt>uid</tt> and adds it to
    # the IP Connection <tt>ipcon</tt>.
    def initialize(uid, ipcon)
      super uid, ipcon, DEVICE_IDENTIFIER, DEVICE_DISPLAY_NAME

      @api_version = [2, 0, 3]

      @response_expected[FUNCTION_SET_VELOCITY] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_GET_VELOCITY] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_GET_CURRENT_VELOCITY] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_SET_ACCELERATION] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_GET_ACCELERATION] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_SET_PWM_FREQUENCY] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_GET_PWM_FREQUENCY] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_FULL_BRAKE] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_GET_STACK_INPUT_VOLTAGE] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_GET_EXTERNAL_INPUT_VOLTAGE] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_GET_CURRENT_CONSUMPTION] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_ENABLE] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_DISABLE] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_IS_ENABLED] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_SET_MINIMUM_VOLTAGE] = RESPONSE_EXPECTED_TRUE
      @response_expected[FUNCTION_GET_MINIMUM_VOLTAGE] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_SET_DRIVE_MODE] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_GET_DRIVE_MODE] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_SET_CURRENT_VELOCITY_PERIOD] = RESPONSE_EXPECTED_TRUE
      @response_expected[FUNCTION_GET_CURRENT_VELOCITY_PERIOD] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_SET_SPITFP_BAUDRATE_CONFIG] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_GET_SPITFP_BAUDRATE_CONFIG] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_GET_SEND_TIMEOUT_COUNT] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_SET_SPITFP_BAUDRATE] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_GET_SPITFP_BAUDRATE] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_GET_SPITFP_ERROR_COUNT] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_ENABLE_STATUS_LED] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_DISABLE_STATUS_LED] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_IS_STATUS_LED_ENABLED] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_GET_PROTOCOL1_BRICKLET_NAME] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_GET_CHIP_TEMPERATURE] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_RESET] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_WRITE_BRICKLET_PLUGIN] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_READ_BRICKLET_PLUGIN] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_GET_IDENTITY] = RESPONSE_EXPECTED_ALWAYS_TRUE

      @callback_formats[CALLBACK_UNDER_VOLTAGE] = [10, 'S']
      @callback_formats[CALLBACK_EMERGENCY_SHUTDOWN] = [8, '']
      @callback_formats[CALLBACK_VELOCITY_REACHED] = [10, 's']
      @callback_formats[CALLBACK_CURRENT_VELOCITY] = [10, 's']

      @ipcon.add_device self
    end

    # Sets the velocity of the motor. Whereas -32767 is full speed backward,
    # 0 is stop and 32767 is full speed forward. Depending on the
    # acceleration (see BrickDC#set_acceleration), the motor is not immediately
    # brought to the velocity but smoothly accelerated.
    # 
    # The velocity describes the duty cycle of the PWM with which the motor is
    # controlled, e.g. a velocity of 3277 sets a PWM with a 10% duty cycle.
    # You can not only control the duty cycle of the PWM but also the frequency,
    # see BrickDC#set_pwm_frequency.
    def set_velocity(velocity)
      check_validity

      send_request FUNCTION_SET_VELOCITY, [velocity], 's', 8, ''
    end

    # Returns the velocity as set by BrickDC#set_velocity.
    def get_velocity
      check_validity

      send_request FUNCTION_GET_VELOCITY, [], '', 10, 's'
    end

    # Returns the *current* velocity of the motor. This value is different
    # from BrickDC#get_velocity whenever the motor is currently accelerating
    # to a goal set by BrickDC#set_velocity.
    def get_current_velocity
      check_validity

      send_request FUNCTION_GET_CURRENT_VELOCITY, [], '', 10, 's'
    end

    # Sets the acceleration of the motor. It is given in *velocity/s*. An
    # acceleration of 10000 means, that every second the velocity is increased
    # by 10000 (or about 30% duty cycle).
    # 
    # For example: If the current velocity is 0 and you want to accelerate to a
    # velocity of 16000 (about 50% duty cycle) in 10 seconds, you should set
    # an acceleration of 1600.
    # 
    # If acceleration is set to 0, there is no speed ramping, i.e. a new velocity
    # is immediately given to the motor.
    def set_acceleration(acceleration)
      check_validity

      send_request FUNCTION_SET_ACCELERATION, [acceleration], 'S', 8, ''
    end

    # Returns the acceleration as set by BrickDC#set_acceleration.
    def get_acceleration
      check_validity

      send_request FUNCTION_GET_ACCELERATION, [], '', 10, 'S'
    end

    # Sets the frequency of the PWM with which the motor is driven.
    # Often a high frequency
    # is less noisy and the motor runs smoother. However, with a low frequency
    # there are less switches and therefore fewer switching losses. Also with
    # most motors lower frequencies enable higher torque.
    # 
    # If you have no idea what all this means, just ignore this function and use
    # the default frequency, it will very likely work fine.
    def set_pwm_frequency(frequency)
      check_validity

      send_request FUNCTION_SET_PWM_FREQUENCY, [frequency], 'S', 8, ''
    end

    # Returns the PWM frequency as set by BrickDC#set_pwm_frequency.
    def get_pwm_frequency
      check_validity

      send_request FUNCTION_GET_PWM_FREQUENCY, [], '', 10, 'S'
    end

    # Executes an active full brake.
    # 
    # .. warning::
    #  This function is for emergency purposes,
    #  where an immediate brake is necessary. Depending on the current velocity and
    #  the strength of the motor, a full brake can be quite violent.
    # 
    # Call BrickDC#set_velocity with 0 if you just want to stop the motor.
    def full_brake
      check_validity

      send_request FUNCTION_FULL_BRAKE, [], '', 8, ''
    end

    # Returns the stack input voltage. The stack input voltage is the
    # voltage that is supplied via the stack, i.e. it is given by a
    # Step-Down or Step-Up Power Supply.
    def get_stack_input_voltage
      check_validity

      send_request FUNCTION_GET_STACK_INPUT_VOLTAGE, [], '', 10, 'S'
    end

    # Returns the external input voltage. The external input voltage is
    # given via the black power input connector on the DC Brick.
    # 
    # If there is an external input voltage and a stack input voltage, the motor
    # will be driven by the external input voltage. If there is only a stack
    # voltage present, the motor will be driven by this voltage.
    # 
    # .. warning::
    #  This means, if you have a high stack voltage and a low external voltage,
    #  the motor will be driven with the low external voltage. If you then remove
    #  the external connection, it will immediately be driven by the high
    #  stack voltage.
    def get_external_input_voltage
      check_validity

      send_request FUNCTION_GET_EXTERNAL_INPUT_VOLTAGE, [], '', 10, 'S'
    end

    # Returns the current consumption of the motor.
    def get_current_consumption
      check_validity

      send_request FUNCTION_GET_CURRENT_CONSUMPTION, [], '', 10, 'S'
    end

    # Enables the driver chip. The driver parameters can be configured (velocity,
    # acceleration, etc) before it is enabled.
    def enable
      check_validity

      send_request FUNCTION_ENABLE, [], '', 8, ''
    end

    # Disables the driver chip. The configurations are kept (velocity,
    # acceleration, etc) but the motor is not driven until it is enabled again.
    # 
    # .. warning::
    #  Disabling the driver chip while the motor is still turning can damage the
    #  driver chip. The motor should be stopped calling BrickDC#set_velocity with 0
    #  before disabling the motor power. The BrickDC#set_velocity function will **not**
    #  wait until the motor is actually stopped. You have to explicitly wait for the
    #  appropriate time after calling the BrickDC#set_velocity function before calling
    #  the BrickDC#disable function.
    def disable
      check_validity

      send_request FUNCTION_DISABLE, [], '', 8, ''
    end

    # Returns *true* if the driver chip is enabled, *false* otherwise.
    def is_enabled
      check_validity

      send_request FUNCTION_IS_ENABLED, [], '', 9, '?'
    end

    # Sets the minimum voltage, below which the CALLBACK_UNDER_VOLTAGE callback
    # is triggered. The minimum possible value that works with the DC Brick is 6V.
    # You can use this function to detect the discharge of a battery that is used
    # to drive the motor. If you have a fixed power supply, you likely do not need
    # this functionality.
    def set_minimum_voltage(voltage)
      check_validity

      send_request FUNCTION_SET_MINIMUM_VOLTAGE, [voltage], 'S', 8, ''
    end

    # Returns the minimum voltage as set by BrickDC#set_minimum_voltage
    def get_minimum_voltage
      check_validity

      send_request FUNCTION_GET_MINIMUM_VOLTAGE, [], '', 10, 'S'
    end

    # Sets the drive mode. Possible modes are:
    # 
    # * 0 = Drive/Brake
    # * 1 = Drive/Coast
    # 
    # These modes are different kinds of motor controls.
    # 
    # In Drive/Brake mode, the motor is always either driving or braking. There
    # is no freewheeling. Advantages are: A more linear correlation between
    # PWM and velocity, more exact accelerations and the possibility to drive
    # with slower velocities.
    # 
    # In Drive/Coast mode, the motor is always either driving or freewheeling.
    # Advantages are: Less current consumption and less demands on the motor and
    # driver chip.
    def set_drive_mode(mode)
      check_validity

      send_request FUNCTION_SET_DRIVE_MODE, [mode], 'C', 8, ''
    end

    # Returns the drive mode, as set by BrickDC#set_drive_mode.
    def get_drive_mode
      check_validity

      send_request FUNCTION_GET_DRIVE_MODE, [], '', 9, 'C'
    end

    # Sets a period with which the CALLBACK_CURRENT_VELOCITY callback is triggered.
    # A period of 0 turns the callback off.
    def set_current_velocity_period(period)
      check_validity

      send_request FUNCTION_SET_CURRENT_VELOCITY_PERIOD, [period], 'S', 8, ''
    end

    # Returns the period as set by BrickDC#set_current_velocity_period.
    def get_current_velocity_period
      check_validity

      send_request FUNCTION_GET_CURRENT_VELOCITY_PERIOD, [], '', 10, 'S'
    end

    # The SPITF protocol can be used with a dynamic baudrate. If the dynamic baudrate is
    # enabled, the Brick will try to adapt the baudrate for the communication
    # between Bricks and Bricklets according to the amount of data that is transferred.
    # 
    # The baudrate will be increased exponentially if lots of data is sent/received and
    # decreased linearly if little data is sent/received.
    # 
    # This lowers the baudrate in applications where little data is transferred (e.g.
    # a weather station) and increases the robustness. If there is lots of data to transfer
    # (e.g. Thermal Imaging Bricklet) it automatically increases the baudrate as needed.
    # 
    # In cases where some data has to transferred as fast as possible every few seconds
    # (e.g. RS485 Bricklet with a high baudrate but small payload) you may want to turn
    # the dynamic baudrate off to get the highest possible performance.
    # 
    # The maximum value of the baudrate can be set per port with the function
    # BrickDC#set_spitfp_baudrate. If the dynamic baudrate is disabled, the baudrate
    # as set by BrickDC#set_spitfp_baudrate will be used statically.
    # 
    # .. versionadded:: 2.3.5$nbsp;(Firmware)
    def set_spitfp_baudrate_config(enable_dynamic_baudrate, minimum_dynamic_baudrate)
      check_validity

      send_request FUNCTION_SET_SPITFP_BAUDRATE_CONFIG, [enable_dynamic_baudrate, minimum_dynamic_baudrate], '? L', 8, ''
    end

    # Returns the baudrate config, see BrickDC#set_spitfp_baudrate_config.
    # 
    # .. versionadded:: 2.3.5$nbsp;(Firmware)
    def get_spitfp_baudrate_config
      check_validity

      send_request FUNCTION_GET_SPITFP_BAUDRATE_CONFIG, [], '', 13, '? L'
    end

    # Returns the timeout count for the different communication methods.
    # 
    # The methods 0-2 are available for all Bricks, 3-7 only for Master Bricks.
    # 
    # This function is mostly used for debugging during development, in normal operation
    # the counters should nearly always stay at 0.
    # 
    # .. versionadded:: 2.3.3$nbsp;(Firmware)
    def get_send_timeout_count(communication_method)
      check_validity

      send_request FUNCTION_GET_SEND_TIMEOUT_COUNT, [communication_method], 'C', 12, 'L'
    end

    # Sets the baudrate for a specific Bricklet port.
    # 
    # If you want to increase the throughput of Bricklets you can increase
    # the baudrate. If you get a high error count because of high
    # interference (see BrickDC#get_spitfp_error_count) you can decrease the
    # baudrate.
    # 
    # If the dynamic baudrate feature is enabled, the baudrate set by this
    # function corresponds to the maximum baudrate (see BrickDC#set_spitfp_baudrate_config).
    # 
    # Regulatory testing is done with the default baudrate. If CE compatibility
    # or similar is necessary in your applications we recommend to not change
    # the baudrate.
    # 
    # .. versionadded:: 2.3.3$nbsp;(Firmware)
    def set_spitfp_baudrate(bricklet_port, baudrate)
      check_validity

      send_request FUNCTION_SET_SPITFP_BAUDRATE, [bricklet_port, baudrate], 'k L', 8, ''
    end

    # Returns the baudrate for a given Bricklet port, see BrickDC#set_spitfp_baudrate.
    # 
    # .. versionadded:: 2.3.3$nbsp;(Firmware)
    def get_spitfp_baudrate(bricklet_port)
      check_validity

      send_request FUNCTION_GET_SPITFP_BAUDRATE, [bricklet_port], 'k', 12, 'L'
    end

    # Returns the error count for the communication between Brick and Bricklet.
    # 
    # The errors are divided into
    # 
    # * ACK checksum errors,
    # * message checksum errors,
    # * framing errors and
    # * overflow errors.
    # 
    # The errors counts are for errors that occur on the Brick side. All
    # Bricklets have a similar function that returns the errors on the Bricklet side.
    # 
    # .. versionadded:: 2.3.3$nbsp;(Firmware)
    def get_spitfp_error_count(bricklet_port)
      check_validity

      send_request FUNCTION_GET_SPITFP_ERROR_COUNT, [bricklet_port], 'k', 24, 'L L L L'
    end

    # Enables the status LED.
    # 
    # The status LED is the blue LED next to the USB connector. If enabled is is
    # on and it flickers if data is transfered. If disabled it is always off.
    # 
    # The default state is enabled.
    # 
    # .. versionadded:: 2.3.1$nbsp;(Firmware)
    def enable_status_led
      check_validity

      send_request FUNCTION_ENABLE_STATUS_LED, [], '', 8, ''
    end

    # Disables the status LED.
    # 
    # The status LED is the blue LED next to the USB connector. If enabled is is
    # on and it flickers if data is transfered. If disabled it is always off.
    # 
    # The default state is enabled.
    # 
    # .. versionadded:: 2.3.1$nbsp;(Firmware)
    def disable_status_led
      check_validity

      send_request FUNCTION_DISABLE_STATUS_LED, [], '', 8, ''
    end

    # Returns *true* if the status LED is enabled, *false* otherwise.
    # 
    # .. versionadded:: 2.3.1$nbsp;(Firmware)
    def is_status_led_enabled
      check_validity

      send_request FUNCTION_IS_STATUS_LED_ENABLED, [], '', 9, '?'
    end

    # Returns the firmware and protocol version and the name of the Bricklet for a
    # given port.
    # 
    # This functions sole purpose is to allow automatic flashing of v1.x.y Bricklet
    # plugins.
    def get_protocol1_bricklet_name(port)
      check_validity

      send_request FUNCTION_GET_PROTOCOL1_BRICKLET_NAME, [port], 'k', 52, 'C C3 Z40'
    end

    # Returns the temperature as measured inside the microcontroller. The
    # value returned is not the ambient temperature!
    # 
    # The temperature is only proportional to the real temperature and it has an
    # accuracy of ±15%. Practically it is only useful as an indicator for
    # temperature changes.
    def get_chip_temperature
      check_validity

      send_request FUNCTION_GET_CHIP_TEMPERATURE, [], '', 10, 's'
    end

    # Calling this function will reset the Brick. Calling this function
    # on a Brick inside of a stack will reset the whole stack.
    # 
    # After a reset you have to create new device objects,
    # calling functions on the existing ones will result in
    # undefined behavior!
    def reset
      check_validity

      send_request FUNCTION_RESET, [], '', 8, ''
    end

    # Writes 32 bytes of firmware to the bricklet attached at the given port.
    # The bytes are written to the position offset * 32.
    # 
    # This function is used by Brick Viewer during flashing. It should not be
    # necessary to call it in a normal user program.
    def write_bricklet_plugin(port, offset, chunk)
      check_validity

      send_request FUNCTION_WRITE_BRICKLET_PLUGIN, [port, offset, chunk], 'k C C32', 8, ''
    end

    # Reads 32 bytes of firmware from the bricklet attached at the given port.
    # The bytes are read starting at the position offset * 32.
    # 
    # This function is used by Brick Viewer during flashing. It should not be
    # necessary to call it in a normal user program.
    def read_bricklet_plugin(port, offset)
      check_validity

      send_request FUNCTION_READ_BRICKLET_PLUGIN, [port, offset], 'k C', 40, 'C32'
    end

    # Returns the UID, the UID where the Brick is connected to,
    # the position, the hardware and firmware version as well as the
    # device identifier.
    # 
    # The position is the position in the stack from '0' (bottom) to '8' (top).
    # 
    # The device identifier numbers can be found :ref:`here <device_identifier>`.
    # |device_identifier_constant|
    def get_identity
      send_request FUNCTION_GET_IDENTITY, [], '', 33, 'Z8 Z8 k C3 C3 S'
    end

    # Registers a callback with ID <tt>id</tt> to the block <tt>block</tt>.
    def register_callback(id, &block)
      callback = block
      @registered_callbacks[id] = callback
    end
  end
end
