# -*- ruby encoding: utf-8 -*-
#############################################################
# This file was automatically generated on 2021-05-06.      #
#                                                           #
# Ruby Bindings Version 2.1.29                              #
#                                                           #
# If you have a bugfix for this file and want to commit it, #
# please fix the bug in the generator. You can find a link  #
# to the generators git repository on tinkerforge.com       #
#############################################################

require_relative './ip_connection'

module Tinkerforge
  # Communicates with CAN bus devices
  class BrickletCAN < Device
    DEVICE_IDENTIFIER = 270 # :nodoc:
    DEVICE_DISPLAY_NAME = 'CAN Bricklet' # :nodoc:

    # This callback is triggered if a data or remote frame was received by the CAN
    # transceiver.
    # 
    # The ``identifier`` return value follows the identifier format described for
    # BrickletCAN#write_frame.
    # 
    # For remote frames the ``data`` return value always contains invalid values.
    # 
    # A configurable read filter can be used to define which frames should be
    # received by the CAN transceiver at all (see BrickletCAN#set_read_filter).
    # 
    # To enable this callback, use BrickletCAN#enable_frame_read_callback.
    CALLBACK_FRAME_READ = 11

    # This callback is triggered if a data or remote frame was received by the CAN
    # transceiver. The received frame can be read with BrickletCAN#read_frame.
    # If additional frames are received, but BrickletCAN#read_frame was not called yet, the callback
    # will not trigger again.
    # 
    # A configurable read filter can be used to define which frames should be
    # received by the CAN transceiver and put into the read queue (see
    # BrickletCAN#set_read_filter).
    # 
    # To enable this callback, use BrickletCAN#set_frame_readable_callback_configuration.
    # 
    # .. versionadded:: 2.0.1$nbsp;(Plugin)
    CALLBACK_FRAME_READABLE = 14

    FUNCTION_WRITE_FRAME = 1 # :nodoc:
    FUNCTION_READ_FRAME = 2 # :nodoc:
    FUNCTION_ENABLE_FRAME_READ_CALLBACK = 3 # :nodoc:
    FUNCTION_DISABLE_FRAME_READ_CALLBACK = 4 # :nodoc:
    FUNCTION_IS_FRAME_READ_CALLBACK_ENABLED = 5 # :nodoc:
    FUNCTION_SET_CONFIGURATION = 6 # :nodoc:
    FUNCTION_GET_CONFIGURATION = 7 # :nodoc:
    FUNCTION_SET_READ_FILTER = 8 # :nodoc:
    FUNCTION_GET_READ_FILTER = 9 # :nodoc:
    FUNCTION_GET_ERROR_LOG = 10 # :nodoc:
    FUNCTION_SET_FRAME_READABLE_CALLBACK_CONFIGURATION = 12 # :nodoc:
    FUNCTION_GET_FRAME_READABLE_CALLBACK_CONFIGURATION = 13 # :nodoc:
    FUNCTION_GET_IDENTITY = 255 # :nodoc:

    FRAME_TYPE_STANDARD_DATA = 0 # :nodoc:
    FRAME_TYPE_STANDARD_REMOTE = 1 # :nodoc:
    FRAME_TYPE_EXTENDED_DATA = 2 # :nodoc:
    FRAME_TYPE_EXTENDED_REMOTE = 3 # :nodoc:
    BAUD_RATE_10KBPS = 0 # :nodoc:
    BAUD_RATE_20KBPS = 1 # :nodoc:
    BAUD_RATE_50KBPS = 2 # :nodoc:
    BAUD_RATE_125KBPS = 3 # :nodoc:
    BAUD_RATE_250KBPS = 4 # :nodoc:
    BAUD_RATE_500KBPS = 5 # :nodoc:
    BAUD_RATE_800KBPS = 6 # :nodoc:
    BAUD_RATE_1000KBPS = 7 # :nodoc:
    TRANSCEIVER_MODE_NORMAL = 0 # :nodoc:
    TRANSCEIVER_MODE_LOOPBACK = 1 # :nodoc:
    TRANSCEIVER_MODE_READ_ONLY = 2 # :nodoc:
    FILTER_MODE_DISABLED = 0 # :nodoc:
    FILTER_MODE_ACCEPT_ALL = 1 # :nodoc:
    FILTER_MODE_MATCH_STANDARD = 2 # :nodoc:
    FILTER_MODE_MATCH_STANDARD_AND_DATA = 3 # :nodoc:
    FILTER_MODE_MATCH_EXTENDED = 4 # :nodoc:

    # Creates an object with the unique device ID <tt>uid</tt> and adds it to
    # the IP Connection <tt>ipcon</tt>.
    def initialize(uid, ipcon)
      super uid, ipcon, DEVICE_IDENTIFIER, DEVICE_DISPLAY_NAME

      @api_version = [2, 0, 1]

      @response_expected[FUNCTION_WRITE_FRAME] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_READ_FRAME] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_ENABLE_FRAME_READ_CALLBACK] = RESPONSE_EXPECTED_TRUE
      @response_expected[FUNCTION_DISABLE_FRAME_READ_CALLBACK] = RESPONSE_EXPECTED_TRUE
      @response_expected[FUNCTION_IS_FRAME_READ_CALLBACK_ENABLED] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_SET_CONFIGURATION] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_GET_CONFIGURATION] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_SET_READ_FILTER] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_GET_READ_FILTER] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_GET_ERROR_LOG] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_SET_FRAME_READABLE_CALLBACK_CONFIGURATION] = RESPONSE_EXPECTED_TRUE
      @response_expected[FUNCTION_GET_FRAME_READABLE_CALLBACK_CONFIGURATION] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_GET_IDENTITY] = RESPONSE_EXPECTED_ALWAYS_TRUE

      @callback_formats[CALLBACK_FRAME_READ] = [22, 'C L C8 C']
      @callback_formats[CALLBACK_FRAME_READABLE] = [8, '']

      @ipcon.add_device self
    end

    # Writes a data or remote frame to the write buffer to be transmitted over the
    # CAN transceiver.
    # 
    # The Bricklet supports the standard 11-bit (CAN 2.0A) and the additional extended
    # 18-bit (CAN 2.0B) identifiers. For standard frames the Bricklet uses bit 0 to 10
    # from the ``identifier`` parameter as standard 11-bit identifier. For extended
    # frames the Bricklet additionally uses bit 11 to 28 from the ``identifier``
    # parameter as extended 18-bit identifier.
    # 
    # For remote frames the ``data`` parameter is ignored.
    # 
    # Returns *true* if the frame was successfully added to the write buffer. Returns
    # *false* if the frame could not be added because write buffer is already full.
    # 
    # The write buffer can overflow if frames are written to it at a higher rate
    # than the Bricklet can transmitted them over the CAN transceiver. This may
    # happen if the CAN transceiver is configured as read-only or is using a low baud
    # rate (see BrickletCAN#set_configuration). It can also happen if the CAN bus is
    # congested and the frame cannot be transmitted because it constantly loses
    # arbitration or because the CAN transceiver is currently disabled due to a high
    # write error level (see BrickletCAN#get_error_log).
    def write_frame(frame_type, identifier, data, length)
      check_validity

      send_request FUNCTION_WRITE_FRAME, [frame_type, identifier, data, length], 'C L C8 C', 9, '?'
    end

    # Tries to read the next data or remote frame from the read buffer and return it.
    # If a frame was successfully read, then the ``success`` return value is set to
    # *true* and the other return values contain the frame. If the read buffer is
    # empty and no frame could be read, then the ``success`` return value is set to
    # *false* and the other return values contain invalid data.
    # 
    # The ``identifier`` return value follows the identifier format described for
    # BrickletCAN#write_frame.
    # 
    # For remote frames the ``data`` return value always contains invalid data.
    # 
    # A configurable read filter can be used to define which frames should be
    # received by the CAN transceiver and put into the read buffer (see
    # BrickletCAN#set_read_filter).
    # 
    # Instead of polling with this function, you can also use callbacks. See the
    # BrickletCAN#enable_frame_read_callback function and the CALLBACK_FRAME_READ callback.
    def read_frame
      check_validity

      send_request FUNCTION_READ_FRAME, [], '', 23, '? C L C8 C'
    end

    # Enables the CALLBACK_FRAME_READ callback.
    # 
    # By default the callback is disabled. Enabling this callback will disable the CALLBACK_FRAME_READABLE callback.
    def enable_frame_read_callback
      check_validity

      send_request FUNCTION_ENABLE_FRAME_READ_CALLBACK, [], '', 8, ''
    end

    # Disables the CALLBACK_FRAME_READ callback.
    # 
    # By default the callback is disabled.
    def disable_frame_read_callback
      check_validity

      send_request FUNCTION_DISABLE_FRAME_READ_CALLBACK, [], '', 8, ''
    end

    # Returns *true* if the CALLBACK_FRAME_READ callback is enabled, *false* otherwise.
    def is_frame_read_callback_enabled
      check_validity

      send_request FUNCTION_IS_FRAME_READ_CALLBACK_ENABLED, [], '', 9, '?'
    end

    # Sets the configuration for the CAN bus communication.
    # 
    # The baud rate can be configured in steps between 10 and 1000 kbit/s.
    # 
    # The CAN transceiver has three different modes:
    # 
    # * Normal: Reads from and writes to the CAN bus and performs active bus
    #   error detection and acknowledgement.
    # * Loopback: All reads and writes are performed internally. The transceiver
    #   is disconnected from the actual CAN bus.
    # * Read-Only: Only reads from the CAN bus, but does neither active bus error
    #   detection nor acknowledgement. Only the receiving part of the transceiver
    #   is connected to the CAN bus.
    # 
    # The write timeout has three different modes that define how a failed frame
    # transmission should be handled:
    # 
    # * One-Shot (= -1): Only one transmission attempt will be made. If the
    #   transmission fails then the frame is discarded.
    # * Infinite (= 0): Infinite transmission attempts will be made. The frame will
    #   never be discarded.
    # * Milliseconds (> 0): A limited number of transmission attempts will be made.
    #   If the frame could not be transmitted successfully after the configured
    #   number of milliseconds then the frame is discarded.
    def set_configuration(baud_rate, transceiver_mode, write_timeout)
      check_validity

      send_request FUNCTION_SET_CONFIGURATION, [baud_rate, transceiver_mode, write_timeout], 'C C l', 8, ''
    end

    # Returns the configuration as set by BrickletCAN#set_configuration.
    def get_configuration
      check_validity

      send_request FUNCTION_GET_CONFIGURATION, [], '', 14, 'C C l'
    end

    # Set the read filter configuration. This can be used to define which frames
    # should be received by the CAN transceiver and put into the read buffer.
    # 
    # The read filter has five different modes that define if and how the mask and
    # the two filters are applied:
    # 
    # * Disabled: No filtering is applied at all. All frames are received even
    #   incomplete and defective frames. This mode should be used for debugging only.
    # * Accept-All: All complete and error-free frames are received.
    # * Match-Standard: Only standard frames with a matching identifier are received.
    # * Match-Standard-and-Data: Only standard frames with matching identifier and
    #   data bytes are received.
    # * Match-Extended: Only extended frames with a matching identifier are received.
    # 
    # The mask and filters are used as bit masks. Their usage depends on the mode:
    # 
    # * Disabled: Mask and filters are ignored.
    # * Accept-All: Mask and filters are ignored.
    # * Match-Standard: Bit 0 to 10 (11 bits) of mask and filters are used to match
    #   the 11-bit identifier of standard frames.
    # * Match-Standard-and-Data: Bit 0 to 10 (11 bits) of mask and filters are used
    #   to match the 11-bit identifier of standard frames. Bit 11 to 18 (8 bits) and
    #   bit 19 to 26 (8 bits) of mask and filters are used to match the first and
    #   second data byte (if present) of standard frames.
    # * Match-Extended: Bit 0 to 10 (11 bits) of mask and filters are used
    #   to match the standard 11-bit identifier part of extended frames. Bit 11 to 28
    #   (18 bits) of mask and filters are used to match the extended 18-bit identifier
    #   part of extended frames.
    # 
    # The mask and filters are applied in this way: The mask is used to select the
    # identifier and data bits that should be compared to the corresponding filter
    # bits. All unselected bits are automatically accepted. All selected bits have
    # to match one of the filters to be accepted. If all bits for the selected mode
    # are accepted then the frame is accepted and is added to the read buffer.
    # 
    #  "Mask Bit", "Filter Bit", "Identifier/Data Bit", "Result"
    # 
    #  0, X, X, Accept
    #  1, 0, 0, Accept
    #  1, 0, 1, Reject
    #  1, 1, 0, Reject
    #  1, 1, 1, Accept
    # 
    # For example, to receive standard frames with identifier 0x123 only the mode can
    # be set to Match-Standard with 0x7FF as mask and 0x123 as filter 1 and filter 2.
    # The mask of 0x7FF selects all 11 identifier bits for matching so that the
    # identifier has to be exactly 0x123 to be accepted.
    # 
    # To accept identifier 0x123 and identifier 0x456 at the same time, just set
    # filter 2 to 0x456 and keep mask and filter 1 unchanged.
    def set_read_filter(mode, mask, filter1, filter2)
      check_validity

      send_request FUNCTION_SET_READ_FILTER, [mode, mask, filter1, filter2], 'C L L L', 8, ''
    end

    # Returns the read filter as set by BrickletCAN#set_read_filter.
    def get_read_filter
      check_validity

      send_request FUNCTION_GET_READ_FILTER, [], '', 21, 'C L L L'
    end

    # Returns information about different kinds of errors.
    # 
    # The write and read error levels indicate the current level of checksum,
    # acknowledgement, form, bit and stuffing errors during CAN bus write and read
    # operations.
    # 
    # When the write error level exceeds 255 then the CAN transceiver gets disabled
    # and no frames can be transmitted or received anymore. The CAN transceiver will
    # automatically be activated again after the CAN bus is idle for a while.
    # 
    # The write and read error levels are not available in read-only transceiver mode
    # (see BrickletCAN#set_configuration) and are reset to 0 as a side effect of changing
    # the configuration or the read filter.
    # 
    # The write timeout, read register and buffer overflow counts represents the
    # number of these errors:
    # 
    # * A write timeout occurs if a frame could not be transmitted before the
    #   configured write timeout expired (see BrickletCAN#set_configuration).
    # * A read register overflow occurs if the read register of the CAN transceiver
    #   still contains the last received frame when the next frame arrives. In this
    #   case the newly arrived frame is lost. This happens if the CAN transceiver
    #   receives more frames than the Bricklet can handle. Using the read filter
    #   (see BrickletCAN#set_read_filter) can help to reduce the amount of received frames.
    #   This count is not exact, but a lower bound, because the Bricklet might not
    #   able detect all overflows if they occur in rapid succession.
    # * A read buffer overflow occurs if the read buffer of the Bricklet is already
    #   full when the next frame should be read from the read register of the CAN
    #   transceiver. In this case the frame in the read register is lost. This
    #   happens if the CAN transceiver receives more frames to be added to the read
    #   buffer than are removed from the read buffer using the BrickletCAN#read_frame
    #   function. Using the CALLBACK_FRAME_READ callback ensures that the read buffer
    #   can not overflow.
    def get_error_log
      check_validity

      send_request FUNCTION_GET_ERROR_LOG, [], '', 23, 'C C ? L L L'
    end

    # Enables/disables the CALLBACK_FRAME_READABLE callback.
    # 
    # By default the callback is disabled. Enabling this callback will disable the CALLBACK_FRAME_READ callback.
    # 
    # .. versionadded:: 2.0.1$nbsp;(Plugin)
    def set_frame_readable_callback_configuration(enabled)
      check_validity

      send_request FUNCTION_SET_FRAME_READABLE_CALLBACK_CONFIGURATION, [enabled], '?', 8, ''
    end

    # Returns *true* if the CALLBACK_FRAME_READABLE callback is enabled, *false* otherwise.
    # 
    # .. versionadded:: 2.0.1$nbsp;(Plugin)
    def get_frame_readable_callback_configuration
      check_validity

      send_request FUNCTION_GET_FRAME_READABLE_CALLBACK_CONFIGURATION, [], '', 9, '?'
    end

    # Returns the UID, the UID where the Bricklet is connected to,
    # the position, the hardware and firmware version as well as the
    # device identifier.
    # 
    # The position can be 'a', 'b', 'c', 'd', 'e', 'f', 'g' or 'h' (Bricklet Port).
    # A Bricklet connected to an :ref:`Isolator Bricklet <isolator_bricklet>` is always at
    # position 'z'.
    # 
    # The device identifier numbers can be found :ref:`here <device_identifier>`.
    # |device_identifier_constant|
    def get_identity
      send_request FUNCTION_GET_IDENTITY, [], '', 33, 'Z8 Z8 k C3 C3 S'
    end

    # Registers a callback with ID <tt>id</tt> to the block <tt>block</tt>.
    def register_callback(id, &block)
      callback = block
      @registered_callbacks[id] = callback
    end
  end
end
