# -*- ruby encoding: utf-8 -*-
#############################################################
# This file was automatically generated on 2021-05-06.      #
#                                                           #
# Ruby Bindings Version 2.1.29                              #
#                                                           #
# If you have a bugfix for this file and want to commit it, #
# please fix the bug in the generator. You can find a link  #
# to the generators git repository on tinkerforge.com       #
#############################################################

require_relative './ip_connection'

module Tinkerforge
  # Two tactile buttons with built-in blue LEDs
  class BrickletDualButton < Device
    DEVICE_IDENTIFIER = 230 # :nodoc:
    DEVICE_DISPLAY_NAME = 'Dual Button Bricklet' # :nodoc:

    # This callback is called whenever a button is pressed.
    # 
    # Possible states for buttons are:
    # 
    # * 0 = pressed
    # * 1 = released
    # 
    # Possible states for LEDs are:
    # 
    # * 0 = AutoToggleOn: Auto toggle enabled and LED on.
    # * 1 = AutoToggleOff: Auto toggle enabled and LED off.
    # * 2 = On: LED on (auto toggle is disabled).
    # * 3 = Off: LED off (auto toggle is disabled).
    CALLBACK_STATE_CHANGED = 4

    FUNCTION_SET_LED_STATE = 1 # :nodoc:
    FUNCTION_GET_LED_STATE = 2 # :nodoc:
    FUNCTION_GET_BUTTON_STATE = 3 # :nodoc:
    FUNCTION_SET_SELECTED_LED_STATE = 5 # :nodoc:
    FUNCTION_GET_IDENTITY = 255 # :nodoc:

    LED_STATE_AUTO_TOGGLE_ON = 0 # :nodoc:
    LED_STATE_AUTO_TOGGLE_OFF = 1 # :nodoc:
    LED_STATE_ON = 2 # :nodoc:
    LED_STATE_OFF = 3 # :nodoc:
    BUTTON_STATE_PRESSED = 0 # :nodoc:
    BUTTON_STATE_RELEASED = 1 # :nodoc:
    LED_LEFT = 0 # :nodoc:
    LED_RIGHT = 1 # :nodoc:

    # Creates an object with the unique device ID <tt>uid</tt> and adds it to
    # the IP Connection <tt>ipcon</tt>.
    def initialize(uid, ipcon)
      super uid, ipcon, DEVICE_IDENTIFIER, DEVICE_DISPLAY_NAME

      @api_version = [2, 0, 0]

      @response_expected[FUNCTION_SET_LED_STATE] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_GET_LED_STATE] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_GET_BUTTON_STATE] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_SET_SELECTED_LED_STATE] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_GET_IDENTITY] = RESPONSE_EXPECTED_ALWAYS_TRUE

      @callback_formats[CALLBACK_STATE_CHANGED] = [12, 'C C C C']

      @ipcon.add_device self
    end

    # Sets the state of the LEDs. Possible states are:
    # 
    # * 0 = AutoToggleOn: Enables auto toggle with initially enabled LED.
    # * 1 = AutoToggleOff: Activates auto toggle with initially disabled LED.
    # * 2 = On: Enables LED (auto toggle is disabled).
    # * 3 = Off: Disables LED (auto toggle is disabled).
    # 
    # In auto toggle mode the LED is toggled automatically at each press of a button.
    # 
    # If you just want to set one of the LEDs and don't know the current state
    # of the other LED, you can get the state with BrickletDualButton#get_led_state or you
    # can use BrickletDualButton#set_selected_led_state.
    def set_led_state(led_l, led_r)
      check_validity

      send_request FUNCTION_SET_LED_STATE, [led_l, led_r], 'C C', 8, ''
    end

    # Returns the current state of the LEDs, as set by BrickletDualButton#set_led_state.
    def get_led_state
      check_validity

      send_request FUNCTION_GET_LED_STATE, [], '', 10, 'C C'
    end

    # Returns the current state for both buttons. Possible states are:
    # 
    # * 0 = pressed
    # * 1 = released
    def get_button_state
      check_validity

      send_request FUNCTION_GET_BUTTON_STATE, [], '', 10, 'C C'
    end

    # Sets the state of the selected LED (0 or 1).
    # 
    # The other LED remains untouched.
    def set_selected_led_state(led, state)
      check_validity

      send_request FUNCTION_SET_SELECTED_LED_STATE, [led, state], 'C C', 8, ''
    end

    # Returns the UID, the UID where the Bricklet is connected to,
    # the position, the hardware and firmware version as well as the
    # device identifier.
    # 
    # The position can be 'a', 'b', 'c', 'd', 'e', 'f', 'g' or 'h' (Bricklet Port).
    # A Bricklet connected to an :ref:`Isolator Bricklet <isolator_bricklet>` is always at
    # position 'z'.
    # 
    # The device identifier numbers can be found :ref:`here <device_identifier>`.
    # |device_identifier_constant|
    def get_identity
      send_request FUNCTION_GET_IDENTITY, [], '', 33, 'Z8 Z8 k C3 C3 S'
    end

    # Registers a callback with ID <tt>id</tt> to the block <tt>block</tt>.
    def register_callback(id, &block)
      callback = block
      @registered_callbacks[id] = callback
    end
  end
end
