# -*- ruby encoding: utf-8 -*-
#############################################################
# This file was automatically generated on 2021-05-06.      #
#                                                           #
# Ruby Bindings Version 2.1.29                              #
#                                                           #
# If you have a bugfix for this file and want to commit it, #
# please fix the bug in the generator. You can find a link  #
# to the generators git repository on tinkerforge.com       #
#############################################################

require_relative './ip_connection'

module Tinkerforge
  # Measures Voltage, Current, Energy, Real/Apparent/Reactive Power, Power Factor and Frequency
  class BrickletEnergyMonitor < Device
    DEVICE_IDENTIFIER = 2152 # :nodoc:
    DEVICE_DISPLAY_NAME = 'Energy Monitor Bricklet' # :nodoc:

    # This callback is triggered periodically according to the configuration set by
    # BrickletEnergyMonitor#set_energy_data_callback_configuration.
    # 
    # The parameters are the same as BrickletEnergyMonitor#get_energy_data.
    CALLBACK_ENERGY_DATA = 10

    FUNCTION_GET_ENERGY_DATA = 1 # :nodoc:
    FUNCTION_RESET_ENERGY = 2 # :nodoc:
    FUNCTION_GET_WAVEFORM_LOW_LEVEL = 3 # :nodoc:
    FUNCTION_GET_TRANSFORMER_STATUS = 4 # :nodoc:
    FUNCTION_SET_TRANSFORMER_CALIBRATION = 5 # :nodoc:
    FUNCTION_GET_TRANSFORMER_CALIBRATION = 6 # :nodoc:
    FUNCTION_CALIBRATE_OFFSET = 7 # :nodoc:
    FUNCTION_SET_ENERGY_DATA_CALLBACK_CONFIGURATION = 8 # :nodoc:
    FUNCTION_GET_ENERGY_DATA_CALLBACK_CONFIGURATION = 9 # :nodoc:
    FUNCTION_GET_SPITFP_ERROR_COUNT = 234 # :nodoc:
    FUNCTION_SET_BOOTLOADER_MODE = 235 # :nodoc:
    FUNCTION_GET_BOOTLOADER_MODE = 236 # :nodoc:
    FUNCTION_SET_WRITE_FIRMWARE_POINTER = 237 # :nodoc:
    FUNCTION_WRITE_FIRMWARE = 238 # :nodoc:
    FUNCTION_SET_STATUS_LED_CONFIG = 239 # :nodoc:
    FUNCTION_GET_STATUS_LED_CONFIG = 240 # :nodoc:
    FUNCTION_GET_CHIP_TEMPERATURE = 242 # :nodoc:
    FUNCTION_RESET = 243 # :nodoc:
    FUNCTION_WRITE_UID = 248 # :nodoc:
    FUNCTION_READ_UID = 249 # :nodoc:
    FUNCTION_GET_IDENTITY = 255 # :nodoc:

    BOOTLOADER_MODE_BOOTLOADER = 0 # :nodoc:
    BOOTLOADER_MODE_FIRMWARE = 1 # :nodoc:
    BOOTLOADER_MODE_BOOTLOADER_WAIT_FOR_REBOOT = 2 # :nodoc:
    BOOTLOADER_MODE_FIRMWARE_WAIT_FOR_REBOOT = 3 # :nodoc:
    BOOTLOADER_MODE_FIRMWARE_WAIT_FOR_ERASE_AND_REBOOT = 4 # :nodoc:
    BOOTLOADER_STATUS_OK = 0 # :nodoc:
    BOOTLOADER_STATUS_INVALID_MODE = 1 # :nodoc:
    BOOTLOADER_STATUS_NO_CHANGE = 2 # :nodoc:
    BOOTLOADER_STATUS_ENTRY_FUNCTION_NOT_PRESENT = 3 # :nodoc:
    BOOTLOADER_STATUS_DEVICE_IDENTIFIER_INCORRECT = 4 # :nodoc:
    BOOTLOADER_STATUS_CRC_MISMATCH = 5 # :nodoc:
    STATUS_LED_CONFIG_OFF = 0 # :nodoc:
    STATUS_LED_CONFIG_ON = 1 # :nodoc:
    STATUS_LED_CONFIG_SHOW_HEARTBEAT = 2 # :nodoc:
    STATUS_LED_CONFIG_SHOW_STATUS = 3 # :nodoc:

    # Creates an object with the unique device ID <tt>uid</tt> and adds it to
    # the IP Connection <tt>ipcon</tt>.
    def initialize(uid, ipcon)
      super uid, ipcon, DEVICE_IDENTIFIER, DEVICE_DISPLAY_NAME

      @api_version = [2, 0, 0]

      @response_expected[FUNCTION_GET_ENERGY_DATA] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_RESET_ENERGY] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_GET_WAVEFORM_LOW_LEVEL] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_GET_TRANSFORMER_STATUS] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_SET_TRANSFORMER_CALIBRATION] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_GET_TRANSFORMER_CALIBRATION] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_CALIBRATE_OFFSET] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_SET_ENERGY_DATA_CALLBACK_CONFIGURATION] = RESPONSE_EXPECTED_TRUE
      @response_expected[FUNCTION_GET_ENERGY_DATA_CALLBACK_CONFIGURATION] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_GET_SPITFP_ERROR_COUNT] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_SET_BOOTLOADER_MODE] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_GET_BOOTLOADER_MODE] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_SET_WRITE_FIRMWARE_POINTER] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_WRITE_FIRMWARE] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_SET_STATUS_LED_CONFIG] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_GET_STATUS_LED_CONFIG] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_GET_CHIP_TEMPERATURE] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_RESET] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_WRITE_UID] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_READ_UID] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_GET_IDENTITY] = RESPONSE_EXPECTED_ALWAYS_TRUE

      @callback_formats[CALLBACK_ENERGY_DATA] = [36, 'l l l l l l S S']

      @ipcon.add_device self
    end

    # Returns all of the measurements that are done by the Energy Monitor Bricklet.
    # 
    # * Voltage RMS
    # * Current RMS
    # * Energy (integrated over time)
    # * Real Power
    # * Apparent Power
    # * Reactive Power
    # * Power Factor
    # * Frequency (AC Frequency of the mains voltage)
    # 
    # The frequency is recalculated every 6 seconds.
    # 
    # All other values are integrated over 10 zero-crossings of the voltage sine wave.
    # With a standard AC mains voltage frequency of 50Hz this results in a 5 measurements
    # per second (or an integration time of 200ms per measurement).
    # 
    # If no voltage transformer is connected, the Bricklet will use the current waveform
    # to calculate the frequency and it will use an integration time of
    # 10 zero-crossings of the current waveform.
    def get_energy_data
      check_validity

      send_request FUNCTION_GET_ENERGY_DATA, [], '', 36, 'l l l l l l S S'
    end

    # Sets the energy value (see BrickletEnergyMonitor#get_energy_data) back to 0Wh.
    def reset_energy
      check_validity

      send_request FUNCTION_RESET_ENERGY, [], '', 8, ''
    end

    # Returns a snapshot of the voltage and current waveform. The values
    # in the returned array alternate between voltage and current. The data from
    # one getter call contains 768 data points for voltage and current, which
    # correspond to about 3 full sine waves.
    # 
    # The voltage is given with a resolution of 100mV and the current is given
    # with a resolution of 10mA.
    # 
    # This data is meant to be used for a non-realtime graphical representation of
    # the voltage and current waveforms.
    def get_waveform_low_level
      check_validity

      send_request FUNCTION_GET_WAVEFORM_LOW_LEVEL, [], '', 70, 'S s30'
    end

    # Returns *true* if a voltage/current transformer is connected to the Bricklet.
    def get_transformer_status
      check_validity

      send_request FUNCTION_GET_TRANSFORMER_STATUS, [], '', 10, '? ?'
    end

    # Sets the transformer ratio for the voltage and current transformer in 1/100 form.
    # 
    # Example: If your mains voltage is 230V, you use 9V voltage transformer and a
    # 1V:30A current clamp your voltage ratio is 230/9 = 25.56 and your current ratio
    # is 30/1 = 30.
    # 
    # In this case you have to set the values 2556 and 3000 for voltage ratio and current
    # ratio.
    # 
    # The calibration is saved in non-volatile memory, you only have to set it once.
    # 
    # Set the phase shift to 0. It is for future use and currently not supported by the Bricklet.
    def set_transformer_calibration(voltage_ratio, current_ratio, phase_shift)
      check_validity

      send_request FUNCTION_SET_TRANSFORMER_CALIBRATION, [voltage_ratio, current_ratio, phase_shift], 'S S s', 8, ''
    end

    # Returns the transformer calibration as set by BrickletEnergyMonitor#set_transformer_calibration.
    def get_transformer_calibration
      check_validity

      send_request FUNCTION_GET_TRANSFORMER_CALIBRATION, [], '', 14, 'S S s'
    end

    # Calling this function will start an offset calibration. The offset calibration will
    # integrate the voltage and current waveform over a longer time period to find the 0
    # transition point in the sine wave.
    # 
    # The Bricklet comes with a factory-calibrated offset value, you should not have to
    # call this function.
    # 
    # If you want to re-calibrate the offset we recommend that you connect a load that
    # has a smooth sinusoidal voltage and current waveform. Alternatively you can also
    # short both inputs.
    # 
    # The calibration is saved in non-volatile memory, you only have to set it once.
    def calibrate_offset
      check_validity

      send_request FUNCTION_CALIBRATE_OFFSET, [], '', 8, ''
    end

    # The period is the period with which the CALLBACK_ENERGY_DATA
    # callback is triggered periodically. A value of 0 turns the callback off.
    # 
    # If the `value has to change`-parameter is set to true, the callback is only
    # triggered after the value has changed. If the value didn't change within the
    # period, the callback is triggered immediately on change.
    # 
    # If it is set to false, the callback is continuously triggered with the period,
    # independent of the value.
    def set_energy_data_callback_configuration(period, value_has_to_change)
      check_validity

      send_request FUNCTION_SET_ENERGY_DATA_CALLBACK_CONFIGURATION, [period, value_has_to_change], 'L ?', 8, ''
    end

    # Returns the callback configuration as set by
    # BrickletEnergyMonitor#set_energy_data_callback_configuration.
    def get_energy_data_callback_configuration
      check_validity

      send_request FUNCTION_GET_ENERGY_DATA_CALLBACK_CONFIGURATION, [], '', 13, 'L ?'
    end

    # Returns the error count for the communication between Brick and Bricklet.
    # 
    # The errors are divided into
    # 
    # * ACK checksum errors,
    # * message checksum errors,
    # * framing errors and
    # * overflow errors.
    # 
    # The errors counts are for errors that occur on the Bricklet side. All
    # Bricks have a similar function that returns the errors on the Brick side.
    def get_spitfp_error_count
      check_validity

      send_request FUNCTION_GET_SPITFP_ERROR_COUNT, [], '', 24, 'L L L L'
    end

    # Sets the bootloader mode and returns the status after the requested
    # mode change was instigated.
    # 
    # You can change from bootloader mode to firmware mode and vice versa. A change
    # from bootloader mode to firmware mode will only take place if the entry function,
    # device identifier and CRC are present and correct.
    # 
    # This function is used by Brick Viewer during flashing. It should not be
    # necessary to call it in a normal user program.
    def set_bootloader_mode(mode)
      check_validity

      send_request FUNCTION_SET_BOOTLOADER_MODE, [mode], 'C', 9, 'C'
    end

    # Returns the current bootloader mode, see BrickletEnergyMonitor#set_bootloader_mode.
    def get_bootloader_mode
      check_validity

      send_request FUNCTION_GET_BOOTLOADER_MODE, [], '', 9, 'C'
    end

    # Sets the firmware pointer for BrickletEnergyMonitor#write_firmware. The pointer has
    # to be increased by chunks of size 64. The data is written to flash
    # every 4 chunks (which equals to one page of size 256).
    # 
    # This function is used by Brick Viewer during flashing. It should not be
    # necessary to call it in a normal user program.
    def set_write_firmware_pointer(pointer)
      check_validity

      send_request FUNCTION_SET_WRITE_FIRMWARE_POINTER, [pointer], 'L', 8, ''
    end

    # Writes 64 Bytes of firmware at the position as written by
    # BrickletEnergyMonitor#set_write_firmware_pointer before. The firmware is written
    # to flash every 4 chunks.
    # 
    # You can only write firmware in bootloader mode.
    # 
    # This function is used by Brick Viewer during flashing. It should not be
    # necessary to call it in a normal user program.
    def write_firmware(data)
      check_validity

      send_request FUNCTION_WRITE_FIRMWARE, [data], 'C64', 9, 'C'
    end

    # Sets the status LED configuration. By default the LED shows
    # communication traffic between Brick and Bricklet, it flickers once
    # for every 10 received data packets.
    # 
    # You can also turn the LED permanently on/off or show a heartbeat.
    # 
    # If the Bricklet is in bootloader mode, the LED is will show heartbeat by default.
    def set_status_led_config(config)
      check_validity

      send_request FUNCTION_SET_STATUS_LED_CONFIG, [config], 'C', 8, ''
    end

    # Returns the configuration as set by BrickletEnergyMonitor#set_status_led_config
    def get_status_led_config
      check_validity

      send_request FUNCTION_GET_STATUS_LED_CONFIG, [], '', 9, 'C'
    end

    # Returns the temperature as measured inside the microcontroller. The
    # value returned is not the ambient temperature!
    # 
    # The temperature is only proportional to the real temperature and it has bad
    # accuracy. Practically it is only useful as an indicator for
    # temperature changes.
    def get_chip_temperature
      check_validity

      send_request FUNCTION_GET_CHIP_TEMPERATURE, [], '', 10, 's'
    end

    # Calling this function will reset the Bricklet. All configurations
    # will be lost.
    # 
    # After a reset you have to create new device objects,
    # calling functions on the existing ones will result in
    # undefined behavior!
    def reset
      check_validity

      send_request FUNCTION_RESET, [], '', 8, ''
    end

    # Writes a new UID into flash. If you want to set a new UID
    # you have to decode the Base58 encoded UID string into an
    # integer first.
    # 
    # We recommend that you use Brick Viewer to change the UID.
    def write_uid(uid)
      check_validity

      send_request FUNCTION_WRITE_UID, [uid], 'L', 8, ''
    end

    # Returns the current UID as an integer. Encode as
    # Base58 to get the usual string version.
    def read_uid
      check_validity

      send_request FUNCTION_READ_UID, [], '', 12, 'L'
    end

    # Returns the UID, the UID where the Bricklet is connected to,
    # the position, the hardware and firmware version as well as the
    # device identifier.
    # 
    # The position can be 'a', 'b', 'c', 'd', 'e', 'f', 'g' or 'h' (Bricklet Port).
    # A Bricklet connected to an :ref:`Isolator Bricklet <isolator_bricklet>` is always at
    # position 'z'.
    # 
    # The device identifier numbers can be found :ref:`here <device_identifier>`.
    # |device_identifier_constant|
    def get_identity
      send_request FUNCTION_GET_IDENTITY, [], '', 33, 'Z8 Z8 k C3 C3 S'
    end

    # Returns a snapshot of the voltage and current waveform. The values
    # in the returned array alternate between voltage and current. The data from
    # one getter call contains 768 data points for voltage and current, which
    # correspond to about 3 full sine waves.
    # 
    # The voltage is given with a resolution of 100mV and the current is given
    # with a resolution of 10mA.
    # 
    # This data is meant to be used for a non-realtime graphical representation of
    # the voltage and current waveforms.
    def get_waveform
      waveform_length = 1536
      waveform_data = nil # assigned in block

      @stream_mutex.synchronize {
        ret = get_waveform_low_level
        waveform_chunk_offset = ret[0]

        if waveform_chunk_offset == (1 << 16) - 1 # maximum chunk offset -> stream has no data
          waveform_length = 0
          waveform_chunk_offset = 0
          waveform_out_of_sync = false
          waveform_data = []
        else
          waveform_out_of_sync = waveform_chunk_offset != 0
          waveform_data = ret[1]
        end

        while not waveform_out_of_sync and waveform_data.length < waveform_length
          ret = get_waveform_low_level
          waveform_chunk_offset = ret[0]
          waveform_out_of_sync = waveform_chunk_offset != waveform_data.length
          waveform_data += ret[1]
        end

        if waveform_out_of_sync # discard remaining stream to bring it back in-sync
          while waveform_chunk_offset + 30 < waveform_length
            ret = get_waveform_low_level
            waveform_chunk_offset = ret[0]
          end

          raise StreamOutOfSyncException, 'Waveform stream is out-of-sync'
        end
      }

      waveform_data[0, waveform_length]
    end

    # Registers a callback with ID <tt>id</tt> to the block <tt>block</tt>.
    def register_callback(id, &block)
      callback = block
      @registered_callbacks[id] = callback
    end
  end
end
