# -*- ruby encoding: utf-8 -*-
#############################################################
# This file was automatically generated on 2021-05-06.      #
#                                                           #
# Ruby Bindings Version 2.1.29                              #
#                                                           #
# If you have a bugfix for this file and want to commit it, #
# please fix the bug in the generator. You can find a link  #
# to the generators git repository on tinkerforge.com       #
#############################################################

require_relative './ip_connection'

module Tinkerforge
  # Generates configurable DC voltage and current, 0V to 10V and 4mA to 20mA
  class BrickletIndustrialAnalogOut < Device
    DEVICE_IDENTIFIER = 258 # :nodoc:
    DEVICE_DISPLAY_NAME = 'Industrial Analog Out Bricklet' # :nodoc:

    FUNCTION_ENABLE = 1 # :nodoc:
    FUNCTION_DISABLE = 2 # :nodoc:
    FUNCTION_IS_ENABLED = 3 # :nodoc:
    FUNCTION_SET_VOLTAGE = 4 # :nodoc:
    FUNCTION_GET_VOLTAGE = 5 # :nodoc:
    FUNCTION_SET_CURRENT = 6 # :nodoc:
    FUNCTION_GET_CURRENT = 7 # :nodoc:
    FUNCTION_SET_CONFIGURATION = 8 # :nodoc:
    FUNCTION_GET_CONFIGURATION = 9 # :nodoc:
    FUNCTION_GET_IDENTITY = 255 # :nodoc:

    VOLTAGE_RANGE_0_TO_5V = 0 # :nodoc:
    VOLTAGE_RANGE_0_TO_10V = 1 # :nodoc:
    CURRENT_RANGE_4_TO_20MA = 0 # :nodoc:
    CURRENT_RANGE_0_TO_20MA = 1 # :nodoc:
    CURRENT_RANGE_0_TO_24MA = 2 # :nodoc:

    # Creates an object with the unique device ID <tt>uid</tt> and adds it to
    # the IP Connection <tt>ipcon</tt>.
    def initialize(uid, ipcon)
      super uid, ipcon, DEVICE_IDENTIFIER, DEVICE_DISPLAY_NAME

      @api_version = [2, 0, 0]

      @response_expected[FUNCTION_ENABLE] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_DISABLE] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_IS_ENABLED] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_SET_VOLTAGE] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_GET_VOLTAGE] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_SET_CURRENT] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_GET_CURRENT] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_SET_CONFIGURATION] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_GET_CONFIGURATION] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_GET_IDENTITY] = RESPONSE_EXPECTED_ALWAYS_TRUE


      @ipcon.add_device self
    end

    # Enables the output of voltage and current.
    # 
    # The default is disabled.
    def enable
      check_validity

      send_request FUNCTION_ENABLE, [], '', 8, ''
    end

    # Disables the output of voltage and current.
    # 
    # The default is disabled.
    def disable
      check_validity

      send_request FUNCTION_DISABLE, [], '', 8, ''
    end

    # Returns *true* if output of voltage and current is enabled, *false* otherwise.
    def is_enabled
      check_validity

      send_request FUNCTION_IS_ENABLED, [], '', 9, '?'
    end

    # Sets the output voltage.
    # 
    # The output voltage and output current are linked. Changing the output voltage
    # also changes the output current.
    def set_voltage(voltage)
      check_validity

      send_request FUNCTION_SET_VOLTAGE, [voltage], 'S', 8, ''
    end

    # Returns the voltage as set by BrickletIndustrialAnalogOut#set_voltage.
    def get_voltage
      check_validity

      send_request FUNCTION_GET_VOLTAGE, [], '', 10, 'S'
    end

    # Sets the output current.
    # 
    # The output current and output voltage are linked. Changing the output current
    # also changes the output voltage.
    def set_current(current)
      check_validity

      send_request FUNCTION_SET_CURRENT, [current], 'S', 8, ''
    end

    # Returns the current as set by BrickletIndustrialAnalogOut#set_current.
    def get_current
      check_validity

      send_request FUNCTION_GET_CURRENT, [], '', 10, 'S'
    end

    # Configures the voltage and current range.
    # 
    # Possible voltage ranges are:
    # 
    # * 0V to 5V
    # * 0V to 10V
    # 
    # Possible current ranges are:
    # 
    # * 4mA to 20mA
    # * 0mA to 20mA
    # * 0mA to 24mA
    # 
    # The resolution will always be 12 bit. This means, that the
    # precision is higher with a smaller range.
    def set_configuration(voltage_range, current_range)
      check_validity

      send_request FUNCTION_SET_CONFIGURATION, [voltage_range, current_range], 'C C', 8, ''
    end

    # Returns the configuration as set by BrickletIndustrialAnalogOut#set_configuration.
    def get_configuration
      check_validity

      send_request FUNCTION_GET_CONFIGURATION, [], '', 10, 'C C'
    end

    # Returns the UID, the UID where the Bricklet is connected to,
    # the position, the hardware and firmware version as well as the
    # device identifier.
    # 
    # The position can be 'a', 'b', 'c', 'd', 'e', 'f', 'g' or 'h' (Bricklet Port).
    # A Bricklet connected to an :ref:`Isolator Bricklet <isolator_bricklet>` is always at
    # position 'z'.
    # 
    # The device identifier numbers can be found :ref:`here <device_identifier>`.
    # |device_identifier_constant|
    def get_identity
      send_request FUNCTION_GET_IDENTITY, [], '', 33, 'Z8 Z8 k C3 C3 S'
    end

  end
end
