# -*- ruby encoding: utf-8 -*-
#############################################################
# This file was automatically generated on 2021-05-06.      #
#                                                           #
# Ruby Bindings Version 2.1.29                              #
#                                                           #
# If you have a bugfix for this file and want to commit it, #
# please fix the bug in the generator. You can find a link  #
# to the generators git repository on tinkerforge.com       #
#############################################################

require_relative './ip_connection'

module Tinkerforge
  # Measures two DC voltages between -35V and +35V with 24bit resolution each
  class BrickletIndustrialDualAnalogIn < Device
    DEVICE_IDENTIFIER = 249 # :nodoc:
    DEVICE_DISPLAY_NAME = 'Industrial Dual Analog In Bricklet' # :nodoc:

    # This callback is triggered periodically with the period that is set by
    # BrickletIndustrialDualAnalogIn#set_voltage_callback_period. The parameter is the voltage of the
    # channel.
    # 
    # The CALLBACK_VOLTAGE callback is only triggered if the voltage has changed since the
    # last triggering.
    CALLBACK_VOLTAGE = 13

    # This callback is triggered when the threshold as set by
    # BrickletIndustrialDualAnalogIn#set_voltage_callback_threshold is reached.
    # The parameter is the voltage of the channel.
    # 
    # If the threshold keeps being reached, the callback is triggered periodically
    # with the period as set by BrickletIndustrialDualAnalogIn#set_debounce_period.
    CALLBACK_VOLTAGE_REACHED = 14

    FUNCTION_GET_VOLTAGE = 1 # :nodoc:
    FUNCTION_SET_VOLTAGE_CALLBACK_PERIOD = 2 # :nodoc:
    FUNCTION_GET_VOLTAGE_CALLBACK_PERIOD = 3 # :nodoc:
    FUNCTION_SET_VOLTAGE_CALLBACK_THRESHOLD = 4 # :nodoc:
    FUNCTION_GET_VOLTAGE_CALLBACK_THRESHOLD = 5 # :nodoc:
    FUNCTION_SET_DEBOUNCE_PERIOD = 6 # :nodoc:
    FUNCTION_GET_DEBOUNCE_PERIOD = 7 # :nodoc:
    FUNCTION_SET_SAMPLE_RATE = 8 # :nodoc:
    FUNCTION_GET_SAMPLE_RATE = 9 # :nodoc:
    FUNCTION_SET_CALIBRATION = 10 # :nodoc:
    FUNCTION_GET_CALIBRATION = 11 # :nodoc:
    FUNCTION_GET_ADC_VALUES = 12 # :nodoc:
    FUNCTION_GET_IDENTITY = 255 # :nodoc:

    THRESHOLD_OPTION_OFF = 'x' # :nodoc:
    THRESHOLD_OPTION_OUTSIDE = 'o' # :nodoc:
    THRESHOLD_OPTION_INSIDE = 'i' # :nodoc:
    THRESHOLD_OPTION_SMALLER = '<' # :nodoc:
    THRESHOLD_OPTION_GREATER = '>' # :nodoc:
    SAMPLE_RATE_976_SPS = 0 # :nodoc:
    SAMPLE_RATE_488_SPS = 1 # :nodoc:
    SAMPLE_RATE_244_SPS = 2 # :nodoc:
    SAMPLE_RATE_122_SPS = 3 # :nodoc:
    SAMPLE_RATE_61_SPS = 4 # :nodoc:
    SAMPLE_RATE_4_SPS = 5 # :nodoc:
    SAMPLE_RATE_2_SPS = 6 # :nodoc:
    SAMPLE_RATE_1_SPS = 7 # :nodoc:

    # Creates an object with the unique device ID <tt>uid</tt> and adds it to
    # the IP Connection <tt>ipcon</tt>.
    def initialize(uid, ipcon)
      super uid, ipcon, DEVICE_IDENTIFIER, DEVICE_DISPLAY_NAME

      @api_version = [2, 0, 0]

      @response_expected[FUNCTION_GET_VOLTAGE] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_SET_VOLTAGE_CALLBACK_PERIOD] = RESPONSE_EXPECTED_TRUE
      @response_expected[FUNCTION_GET_VOLTAGE_CALLBACK_PERIOD] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_SET_VOLTAGE_CALLBACK_THRESHOLD] = RESPONSE_EXPECTED_TRUE
      @response_expected[FUNCTION_GET_VOLTAGE_CALLBACK_THRESHOLD] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_SET_DEBOUNCE_PERIOD] = RESPONSE_EXPECTED_TRUE
      @response_expected[FUNCTION_GET_DEBOUNCE_PERIOD] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_SET_SAMPLE_RATE] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_GET_SAMPLE_RATE] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_SET_CALIBRATION] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_GET_CALIBRATION] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_GET_ADC_VALUES] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_GET_IDENTITY] = RESPONSE_EXPECTED_ALWAYS_TRUE

      @callback_formats[CALLBACK_VOLTAGE] = [13, 'C l']
      @callback_formats[CALLBACK_VOLTAGE_REACHED] = [13, 'C l']

      @ipcon.add_device self
    end

    # Returns the voltage for the given channel.
    # 
    # If you want to get the voltage periodically, it is recommended to use the
    # CALLBACK_VOLTAGE callback and set the period with
    # BrickletIndustrialDualAnalogIn#set_voltage_callback_period.
    def get_voltage(channel)
      check_validity

      send_request FUNCTION_GET_VOLTAGE, [channel], 'C', 12, 'l'
    end

    # Sets the period with which the CALLBACK_VOLTAGE callback is triggered
    # periodically for the given channel. A value of 0 turns the callback off.
    # 
    # The CALLBACK_VOLTAGE callback is only triggered if the voltage has changed since the
    # last triggering.
    def set_voltage_callback_period(channel, period)
      check_validity

      send_request FUNCTION_SET_VOLTAGE_CALLBACK_PERIOD, [channel, period], 'C L', 8, ''
    end

    # Returns the period as set by BrickletIndustrialDualAnalogIn#set_voltage_callback_period.
    def get_voltage_callback_period(channel)
      check_validity

      send_request FUNCTION_GET_VOLTAGE_CALLBACK_PERIOD, [channel], 'C', 12, 'L'
    end

    # Sets the thresholds for the CALLBACK_VOLTAGE_REACHED callback for the given
    # channel.
    # 
    # The following options are possible:
    # 
    #  "Option", "Description"
    # 
    #  "'x'",    "Callback is turned off"
    #  "'o'",    "Callback is triggered when the voltage is *outside* the min and max values"
    #  "'i'",    "Callback is triggered when the voltage is *inside* the min and max values"
    #  "'<'",    "Callback is triggered when the voltage is smaller than the min value (max is ignored)"
    #  "'>'",    "Callback is triggered when the voltage is greater than the min value (max is ignored)"
    def set_voltage_callback_threshold(channel, option, min, max)
      check_validity

      send_request FUNCTION_SET_VOLTAGE_CALLBACK_THRESHOLD, [channel, option, min, max], 'C k l l', 8, ''
    end

    # Returns the threshold as set by BrickletIndustrialDualAnalogIn#set_voltage_callback_threshold.
    def get_voltage_callback_threshold(channel)
      check_validity

      send_request FUNCTION_GET_VOLTAGE_CALLBACK_THRESHOLD, [channel], 'C', 17, 'k l l'
    end

    # Sets the period with which the threshold callback
    # 
    # * CALLBACK_VOLTAGE_REACHED
    # 
    # is triggered, if the threshold
    # 
    # * BrickletIndustrialDualAnalogIn#set_voltage_callback_threshold
    # 
    # keeps being reached.
    def set_debounce_period(debounce)
      check_validity

      send_request FUNCTION_SET_DEBOUNCE_PERIOD, [debounce], 'L', 8, ''
    end

    # Returns the debounce period as set by BrickletIndustrialDualAnalogIn#set_debounce_period.
    def get_debounce_period
      check_validity

      send_request FUNCTION_GET_DEBOUNCE_PERIOD, [], '', 12, 'L'
    end

    # Sets the sample rate. The sample rate can be between 1 sample per second
    # and 976 samples per second. Decreasing the sample rate will also decrease the
    # noise on the data.
    def set_sample_rate(rate)
      check_validity

      send_request FUNCTION_SET_SAMPLE_RATE, [rate], 'C', 8, ''
    end

    # Returns the sample rate as set by BrickletIndustrialDualAnalogIn#set_sample_rate.
    def get_sample_rate
      check_validity

      send_request FUNCTION_GET_SAMPLE_RATE, [], '', 9, 'C'
    end

    # Sets offset and gain of MCP3911 internal calibration registers.
    # 
    # See MCP3911 datasheet 7.7 and 7.8. The Industrial Dual Analog In Bricklet
    # is already factory calibrated by Tinkerforge. It should not be necessary
    # for you to use this function
    def set_calibration(offset, gain)
      check_validity

      send_request FUNCTION_SET_CALIBRATION, [offset, gain], 'l2 l2', 8, ''
    end

    # Returns the calibration as set by BrickletIndustrialDualAnalogIn#set_calibration.
    def get_calibration
      check_validity

      send_request FUNCTION_GET_CALIBRATION, [], '', 24, 'l2 l2'
    end

    # Returns the ADC values as given by the MCP3911 IC. This function
    # is needed for proper calibration, see BrickletIndustrialDualAnalogIn#set_calibration.
    def get_adc_values
      check_validity

      send_request FUNCTION_GET_ADC_VALUES, [], '', 16, 'l2'
    end

    # Returns the UID, the UID where the Bricklet is connected to,
    # the position, the hardware and firmware version as well as the
    # device identifier.
    # 
    # The position can be 'a', 'b', 'c', 'd', 'e', 'f', 'g' or 'h' (Bricklet Port).
    # A Bricklet connected to an :ref:`Isolator Bricklet <isolator_bricklet>` is always at
    # position 'z'.
    # 
    # The device identifier numbers can be found :ref:`here <device_identifier>`.
    # |device_identifier_constant|
    def get_identity
      send_request FUNCTION_GET_IDENTITY, [], '', 33, 'Z8 Z8 k C3 C3 S'
    end

    # Registers a callback with ID <tt>id</tt> to the block <tt>block</tt>.
    def register_callback(id, &block)
      callback = block
      @registered_callbacks[id] = callback
    end
  end
end
