# -*- ruby encoding: utf-8 -*-
#############################################################
# This file was automatically generated on 2021-05-06.      #
#                                                           #
# Ruby Bindings Version 2.1.29                              #
#                                                           #
# If you have a bugfix for this file and want to commit it, #
# please fix the bug in the generator. You can find a link  #
# to the generators git repository on tinkerforge.com       #
#############################################################

require_relative './ip_connection'

module Tinkerforge
  # Measures reflectivity of a surface
  class BrickletLine < Device
    DEVICE_IDENTIFIER = 241 # :nodoc:
    DEVICE_DISPLAY_NAME = 'Line Bricklet' # :nodoc:

    # This callback is triggered periodically with the period that is set by
    # BrickletLine#set_reflectivity_callback_period. The parameter is the
    # reflectivity of the sensor.
    # 
    # The CALLBACK_REFLECTIVITY callback is only triggered if the reflectivity has
    # changed since the last triggering.
    CALLBACK_REFLECTIVITY = 8

    # This callback is triggered when the threshold as set by
    # BrickletLine#set_reflectivity_callback_threshold is reached.
    # The parameter is the reflectivity of the sensor.
    # 
    # If the threshold keeps being reached, the callback is triggered periodically
    # with the period as set by BrickletLine#set_debounce_period.
    CALLBACK_REFLECTIVITY_REACHED = 9

    FUNCTION_GET_REFLECTIVITY = 1 # :nodoc:
    FUNCTION_SET_REFLECTIVITY_CALLBACK_PERIOD = 2 # :nodoc:
    FUNCTION_GET_REFLECTIVITY_CALLBACK_PERIOD = 3 # :nodoc:
    FUNCTION_SET_REFLECTIVITY_CALLBACK_THRESHOLD = 4 # :nodoc:
    FUNCTION_GET_REFLECTIVITY_CALLBACK_THRESHOLD = 5 # :nodoc:
    FUNCTION_SET_DEBOUNCE_PERIOD = 6 # :nodoc:
    FUNCTION_GET_DEBOUNCE_PERIOD = 7 # :nodoc:
    FUNCTION_GET_IDENTITY = 255 # :nodoc:

    THRESHOLD_OPTION_OFF = 'x' # :nodoc:
    THRESHOLD_OPTION_OUTSIDE = 'o' # :nodoc:
    THRESHOLD_OPTION_INSIDE = 'i' # :nodoc:
    THRESHOLD_OPTION_SMALLER = '<' # :nodoc:
    THRESHOLD_OPTION_GREATER = '>' # :nodoc:

    # Creates an object with the unique device ID <tt>uid</tt> and adds it to
    # the IP Connection <tt>ipcon</tt>.
    def initialize(uid, ipcon)
      super uid, ipcon, DEVICE_IDENTIFIER, DEVICE_DISPLAY_NAME

      @api_version = [2, 0, 0]

      @response_expected[FUNCTION_GET_REFLECTIVITY] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_SET_REFLECTIVITY_CALLBACK_PERIOD] = RESPONSE_EXPECTED_TRUE
      @response_expected[FUNCTION_GET_REFLECTIVITY_CALLBACK_PERIOD] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_SET_REFLECTIVITY_CALLBACK_THRESHOLD] = RESPONSE_EXPECTED_TRUE
      @response_expected[FUNCTION_GET_REFLECTIVITY_CALLBACK_THRESHOLD] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_SET_DEBOUNCE_PERIOD] = RESPONSE_EXPECTED_TRUE
      @response_expected[FUNCTION_GET_DEBOUNCE_PERIOD] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_GET_IDENTITY] = RESPONSE_EXPECTED_ALWAYS_TRUE

      @callback_formats[CALLBACK_REFLECTIVITY] = [10, 'S']
      @callback_formats[CALLBACK_REFLECTIVITY_REACHED] = [10, 'S']

      @ipcon.add_device self
    end

    # Returns the currently measured reflectivity. The reflectivity is
    # a value between 0 (not reflective) and 4095 (very reflective).
    # 
    # Usually black has a low reflectivity while white has a high
    # reflectivity.
    # 
    # If you want to get the reflectivity periodically, it is recommended
    # to use the CALLBACK_REFLECTIVITY callback and set the period with
    # BrickletLine#set_reflectivity_callback_period.
    def get_reflectivity
      check_validity

      send_request FUNCTION_GET_REFLECTIVITY, [], '', 10, 'S'
    end

    # Sets the period with which the CALLBACK_REFLECTIVITY callback is triggered
    # periodically. A value of 0 turns the callback off.
    # 
    # The CALLBACK_REFLECTIVITY callback is only triggered if the reflectivity has
    # changed since the last triggering.
    def set_reflectivity_callback_period(period)
      check_validity

      send_request FUNCTION_SET_REFLECTIVITY_CALLBACK_PERIOD, [period], 'L', 8, ''
    end

    # Returns the period as set by BrickletLine#set_reflectivity_callback_period.
    def get_reflectivity_callback_period
      check_validity

      send_request FUNCTION_GET_REFLECTIVITY_CALLBACK_PERIOD, [], '', 12, 'L'
    end

    # Sets the thresholds for the CALLBACK_REFLECTIVITY_REACHED callback.
    # 
    # The following options are possible:
    # 
    #  "Option", "Description"
    # 
    #  "'x'",    "Callback is turned off"
    #  "'o'",    "Callback is triggered when the reflectivity is *outside* the min and max values"
    #  "'i'",    "Callback is triggered when the reflectivity is *inside* the min and max values"
    #  "'<'",    "Callback is triggered when the reflectivity is smaller than the min value (max is ignored)"
    #  "'>'",    "Callback is triggered when the reflectivity is greater than the min value (max is ignored)"
    def set_reflectivity_callback_threshold(option, min, max)
      check_validity

      send_request FUNCTION_SET_REFLECTIVITY_CALLBACK_THRESHOLD, [option, min, max], 'k S S', 8, ''
    end

    # Returns the threshold as set by BrickletLine#set_reflectivity_callback_threshold.
    def get_reflectivity_callback_threshold
      check_validity

      send_request FUNCTION_GET_REFLECTIVITY_CALLBACK_THRESHOLD, [], '', 13, 'k S S'
    end

    # Sets the period with which the threshold callback
    # 
    # * CALLBACK_REFLECTIVITY_REACHED
    # 
    # is triggered, if the threshold
    # 
    # * BrickletLine#set_reflectivity_callback_threshold
    # 
    # keeps being reached.
    def set_debounce_period(debounce)
      check_validity

      send_request FUNCTION_SET_DEBOUNCE_PERIOD, [debounce], 'L', 8, ''
    end

    # Returns the debounce period as set by BrickletLine#set_debounce_period.
    def get_debounce_period
      check_validity

      send_request FUNCTION_GET_DEBOUNCE_PERIOD, [], '', 12, 'L'
    end

    # Returns the UID, the UID where the Bricklet is connected to,
    # the position, the hardware and firmware version as well as the
    # device identifier.
    # 
    # The position can be 'a', 'b', 'c', 'd', 'e', 'f', 'g' or 'h' (Bricklet Port).
    # A Bricklet connected to an :ref:`Isolator Bricklet <isolator_bricklet>` is always at
    # position 'z'.
    # 
    # The device identifier numbers can be found :ref:`here <device_identifier>`.
    # |device_identifier_constant|
    def get_identity
      send_request FUNCTION_GET_IDENTITY, [], '', 33, 'Z8 Z8 k C3 C3 S'
    end

    # Registers a callback with ID <tt>id</tt> to the block <tt>block</tt>.
    def register_callback(id, &block)
      callback = block
      @registered_callbacks[id] = callback
    end
  end
end
