# -*- ruby encoding: utf-8 -*-
#############################################################
# This file was automatically generated on 2021-05-06.      #
#                                                           #
# Ruby Bindings Version 2.1.29                              #
#                                                           #
# If you have a bugfix for this file and want to commit it, #
# please fix the bug in the generator. You can find a link  #
# to the generators git repository on tinkerforge.com       #
#############################################################

require_relative './ip_connection'

module Tinkerforge
  # Measures soil moisture
  class BrickletMoisture < Device
    DEVICE_IDENTIFIER = 232 # :nodoc:
    DEVICE_DISPLAY_NAME = 'Moisture Bricklet' # :nodoc:

    # This callback is triggered periodically with the period that is set by
    # BrickletMoisture#set_moisture_callback_period. The parameter is the
    # moisture value of the sensor.
    # 
    # The CALLBACK_MOISTURE callback is only triggered if the moisture value has changed
    # since the last triggering.
    CALLBACK_MOISTURE = 8

    # This callback is triggered when the threshold as set by
    # BrickletMoisture#set_moisture_callback_threshold is reached.
    # The parameter is the moisture value of the sensor.
    # 
    # If the threshold keeps being reached, the callback is triggered periodically
    # with the period as set by BrickletMoisture#set_debounce_period.
    CALLBACK_MOISTURE_REACHED = 9

    FUNCTION_GET_MOISTURE_VALUE = 1 # :nodoc:
    FUNCTION_SET_MOISTURE_CALLBACK_PERIOD = 2 # :nodoc:
    FUNCTION_GET_MOISTURE_CALLBACK_PERIOD = 3 # :nodoc:
    FUNCTION_SET_MOISTURE_CALLBACK_THRESHOLD = 4 # :nodoc:
    FUNCTION_GET_MOISTURE_CALLBACK_THRESHOLD = 5 # :nodoc:
    FUNCTION_SET_DEBOUNCE_PERIOD = 6 # :nodoc:
    FUNCTION_GET_DEBOUNCE_PERIOD = 7 # :nodoc:
    FUNCTION_SET_MOVING_AVERAGE = 10 # :nodoc:
    FUNCTION_GET_MOVING_AVERAGE = 11 # :nodoc:
    FUNCTION_GET_IDENTITY = 255 # :nodoc:

    THRESHOLD_OPTION_OFF = 'x' # :nodoc:
    THRESHOLD_OPTION_OUTSIDE = 'o' # :nodoc:
    THRESHOLD_OPTION_INSIDE = 'i' # :nodoc:
    THRESHOLD_OPTION_SMALLER = '<' # :nodoc:
    THRESHOLD_OPTION_GREATER = '>' # :nodoc:

    # Creates an object with the unique device ID <tt>uid</tt> and adds it to
    # the IP Connection <tt>ipcon</tt>.
    def initialize(uid, ipcon)
      super uid, ipcon, DEVICE_IDENTIFIER, DEVICE_DISPLAY_NAME

      @api_version = [2, 0, 0]

      @response_expected[FUNCTION_GET_MOISTURE_VALUE] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_SET_MOISTURE_CALLBACK_PERIOD] = RESPONSE_EXPECTED_TRUE
      @response_expected[FUNCTION_GET_MOISTURE_CALLBACK_PERIOD] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_SET_MOISTURE_CALLBACK_THRESHOLD] = RESPONSE_EXPECTED_TRUE
      @response_expected[FUNCTION_GET_MOISTURE_CALLBACK_THRESHOLD] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_SET_DEBOUNCE_PERIOD] = RESPONSE_EXPECTED_TRUE
      @response_expected[FUNCTION_GET_DEBOUNCE_PERIOD] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_SET_MOVING_AVERAGE] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_GET_MOVING_AVERAGE] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_GET_IDENTITY] = RESPONSE_EXPECTED_ALWAYS_TRUE

      @callback_formats[CALLBACK_MOISTURE] = [10, 'S']
      @callback_formats[CALLBACK_MOISTURE_REACHED] = [10, 'S']

      @ipcon.add_device self
    end

    # Returns the current moisture value.
    # A small value corresponds to little moisture, a big
    # value corresponds to much moisture.
    # 
    # If you want to get the moisture value periodically, it is recommended
    # to use the CALLBACK_MOISTURE callback and set the period with
    # BrickletMoisture#set_moisture_callback_period.
    def get_moisture_value
      check_validity

      send_request FUNCTION_GET_MOISTURE_VALUE, [], '', 10, 'S'
    end

    # Sets the period with which the CALLBACK_MOISTURE callback is triggered
    # periodically. A value of 0 turns the callback off.
    # 
    # The CALLBACK_MOISTURE callback is only triggered if the moisture value has changed
    # since the last triggering.
    def set_moisture_callback_period(period)
      check_validity

      send_request FUNCTION_SET_MOISTURE_CALLBACK_PERIOD, [period], 'L', 8, ''
    end

    # Returns the period as set by BrickletMoisture#set_moisture_callback_period.
    def get_moisture_callback_period
      check_validity

      send_request FUNCTION_GET_MOISTURE_CALLBACK_PERIOD, [], '', 12, 'L'
    end

    # Sets the thresholds for the CALLBACK_MOISTURE_REACHED callback.
    # 
    # The following options are possible:
    # 
    #  "Option", "Description"
    # 
    #  "'x'",    "Callback is turned off"
    #  "'o'",    "Callback is triggered when the moisture value is *outside* the min and max values"
    #  "'i'",    "Callback is triggered when the moisture value is *inside* the min and max values"
    #  "'<'",    "Callback is triggered when the moisture value is smaller than the min value (max is ignored)"
    #  "'>'",    "Callback is triggered when the moisture value is greater than the min value (max is ignored)"
    def set_moisture_callback_threshold(option, min, max)
      check_validity

      send_request FUNCTION_SET_MOISTURE_CALLBACK_THRESHOLD, [option, min, max], 'k S S', 8, ''
    end

    # Returns the threshold as set by BrickletMoisture#set_moisture_callback_threshold.
    def get_moisture_callback_threshold
      check_validity

      send_request FUNCTION_GET_MOISTURE_CALLBACK_THRESHOLD, [], '', 13, 'k S S'
    end

    # Sets the period with which the threshold callback
    # 
    # * CALLBACK_MOISTURE_REACHED
    # 
    # is triggered, if the threshold
    # 
    # * BrickletMoisture#set_moisture_callback_threshold
    # 
    # keeps being reached.
    def set_debounce_period(debounce)
      check_validity

      send_request FUNCTION_SET_DEBOUNCE_PERIOD, [debounce], 'L', 8, ''
    end

    # Returns the debounce period as set by BrickletMoisture#set_debounce_period.
    def get_debounce_period
      check_validity

      send_request FUNCTION_GET_DEBOUNCE_PERIOD, [], '', 12, 'L'
    end

    # Sets the length of a `moving averaging <https://en.wikipedia.org/wiki/Moving_average>`__
    # for the moisture value.
    # 
    # Setting the length to 0 will turn the averaging completely off. With less
    # averaging, there is more noise on the data.
    def set_moving_average(average)
      check_validity

      send_request FUNCTION_SET_MOVING_AVERAGE, [average], 'C', 8, ''
    end

    # Returns the length moving average as set by BrickletMoisture#set_moving_average.
    def get_moving_average
      check_validity

      send_request FUNCTION_GET_MOVING_AVERAGE, [], '', 9, 'C'
    end

    # Returns the UID, the UID where the Bricklet is connected to,
    # the position, the hardware and firmware version as well as the
    # device identifier.
    # 
    # The position can be 'a', 'b', 'c', 'd', 'e', 'f', 'g' or 'h' (Bricklet Port).
    # A Bricklet connected to an :ref:`Isolator Bricklet <isolator_bricklet>` is always at
    # position 'z'.
    # 
    # The device identifier numbers can be found :ref:`here <device_identifier>`.
    # |device_identifier_constant|
    def get_identity
      send_request FUNCTION_GET_IDENTITY, [], '', 33, 'Z8 Z8 k C3 C3 S'
    end

    # Registers a callback with ID <tt>id</tt> to the block <tt>block</tt>.
    def register_callback(id, &block)
      callback = block
      @registered_callbacks[id] = callback
    end
  end
end
