# -*- ruby encoding: utf-8 -*-
#############################################################
# This file was automatically generated on 2021-05-06.      #
#                                                           #
# Ruby Bindings Version 2.1.29                              #
#                                                           #
# If you have a bugfix for this file and want to commit it, #
# please fix the bug in the generator. You can find a link  #
# to the generators git repository on tinkerforge.com       #
#############################################################

require_relative './ip_connection'

module Tinkerforge
  # Battery-backed real-time clock
  class BrickletRealTimeClock < Device
    DEVICE_IDENTIFIER = 268 # :nodoc:
    DEVICE_DISPLAY_NAME = 'Real-Time Clock Bricklet' # :nodoc:

    # This callback is triggered periodically with the period that is set by
    # BrickletRealTimeClock#set_date_time_callback_period. The parameters are the same
    # as for BrickletRealTimeClock#get_date_time and BrickletRealTimeClock#get_timestamp combined.
    # 
    # The CALLBACK_DATE_TIME callback is only triggered if the date or time changed
    # since the last triggering.
    # 
    # .. versionadded:: 2.0.1$nbsp;(Plugin)
    CALLBACK_DATE_TIME = 10

    # This callback is triggered every time the current date and time matches the
    # configured alarm (see BrickletRealTimeClock#set_alarm). The parameters are the same
    # as for BrickletRealTimeClock#get_date_time and BrickletRealTimeClock#get_timestamp combined.
    # 
    # .. versionadded:: 2.0.1$nbsp;(Plugin)
    CALLBACK_ALARM = 11

    FUNCTION_SET_DATE_TIME = 1 # :nodoc:
    FUNCTION_GET_DATE_TIME = 2 # :nodoc:
    FUNCTION_GET_TIMESTAMP = 3 # :nodoc:
    FUNCTION_SET_OFFSET = 4 # :nodoc:
    FUNCTION_GET_OFFSET = 5 # :nodoc:
    FUNCTION_SET_DATE_TIME_CALLBACK_PERIOD = 6 # :nodoc:
    FUNCTION_GET_DATE_TIME_CALLBACK_PERIOD = 7 # :nodoc:
    FUNCTION_SET_ALARM = 8 # :nodoc:
    FUNCTION_GET_ALARM = 9 # :nodoc:
    FUNCTION_GET_IDENTITY = 255 # :nodoc:

    WEEKDAY_MONDAY = 1 # :nodoc:
    WEEKDAY_TUESDAY = 2 # :nodoc:
    WEEKDAY_WEDNESDAY = 3 # :nodoc:
    WEEKDAY_THURSDAY = 4 # :nodoc:
    WEEKDAY_FRIDAY = 5 # :nodoc:
    WEEKDAY_SATURDAY = 6 # :nodoc:
    WEEKDAY_SUNDAY = 7 # :nodoc:
    ALARM_MATCH_DISABLED = -1 # :nodoc:
    ALARM_INTERVAL_DISABLED = -1 # :nodoc:

    # Creates an object with the unique device ID <tt>uid</tt> and adds it to
    # the IP Connection <tt>ipcon</tt>.
    def initialize(uid, ipcon)
      super uid, ipcon, DEVICE_IDENTIFIER, DEVICE_DISPLAY_NAME

      @api_version = [2, 0, 1]

      @response_expected[FUNCTION_SET_DATE_TIME] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_GET_DATE_TIME] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_GET_TIMESTAMP] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_SET_OFFSET] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_GET_OFFSET] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_SET_DATE_TIME_CALLBACK_PERIOD] = RESPONSE_EXPECTED_TRUE
      @response_expected[FUNCTION_GET_DATE_TIME_CALLBACK_PERIOD] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_SET_ALARM] = RESPONSE_EXPECTED_TRUE
      @response_expected[FUNCTION_GET_ALARM] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_GET_IDENTITY] = RESPONSE_EXPECTED_ALWAYS_TRUE

      @callback_formats[CALLBACK_DATE_TIME] = [25, 'S C C C C C C C q']
      @callback_formats[CALLBACK_ALARM] = [25, 'S C C C C C C C q']

      @ipcon.add_device self
    end

    # Sets the current date (including weekday) and the current time.
    # 
    # If the backup battery is installed then the real-time clock keeps date and
    # time even if the Bricklet is not powered by a Brick.
    # 
    # The real-time clock handles leap year and inserts the 29th of February
    # accordingly. But leap seconds, time zones and daylight saving time are not
    # handled.
    def set_date_time(year, month, day, hour, minute, second, centisecond, weekday)
      check_validity

      send_request FUNCTION_SET_DATE_TIME, [year, month, day, hour, minute, second, centisecond, weekday], 'S C C C C C C C', 8, ''
    end

    # Returns the current date (including weekday) and the current time of the
    # real-time clock.
    def get_date_time
      check_validity

      send_request FUNCTION_GET_DATE_TIME, [], '', 17, 'S C C C C C C C'
    end

    # Returns the current date and the time of the real-time clock.
    # The timestamp has an effective resolution of hundredths of a
    # second and is an offset to 2000-01-01 00:00:00.000.
    def get_timestamp
      check_validity

      send_request FUNCTION_GET_TIMESTAMP, [], '', 16, 'q'
    end

    # Sets the offset the real-time clock should compensate for in 2.17 ppm steps
    # between -277.76 ppm (-128) and +275.59 ppm (127).
    # 
    # The real-time clock time can deviate from the actual time due to the frequency
    # deviation of its 32.768 kHz crystal. Even without compensation (factory
    # default) the resulting time deviation should be at most ±20 ppm (±52.6
    # seconds per month).
    # 
    # This deviation can be calculated by comparing the same duration measured by the
    # real-time clock (``rtc_duration``) an accurate reference clock
    # (``ref_duration``).
    # 
    # For best results the configured offset should be set to 0 ppm first and then a
    # duration of at least 6 hours should be measured.
    # 
    # The new offset (``new_offset``) can be calculated from the currently configured
    # offset (``current_offset``) and the measured durations as follow::
    # 
    #   new_offset = current_offset - round(1000000 * (rtc_duration - ref_duration) / rtc_duration / 2.17)
    # 
    # If you want to calculate the offset, then we recommend using the calibration
    # dialog in Brick Viewer, instead of doing it manually.
    # 
    # The offset is saved in the EEPROM of the Bricklet and only needs to be
    # configured once.
    def set_offset(offset)
      check_validity

      send_request FUNCTION_SET_OFFSET, [offset], 'c', 8, ''
    end

    # Returns the offset as set by BrickletRealTimeClock#set_offset.
    def get_offset
      check_validity

      send_request FUNCTION_GET_OFFSET, [], '', 9, 'c'
    end

    # Sets the period with which the CALLBACK_DATE_TIME callback is triggered
    # periodically. A value of 0 turns the callback off.
    # 
    # The CALLBACK_DATE_TIME Callback is only triggered if the date or time changed
    # since the last triggering.
    # 
    # .. versionadded:: 2.0.1$nbsp;(Plugin)
    def set_date_time_callback_period(period)
      check_validity

      send_request FUNCTION_SET_DATE_TIME_CALLBACK_PERIOD, [period], 'L', 8, ''
    end

    # Returns the period as set by BrickletRealTimeClock#set_date_time_callback_period.
    # 
    # .. versionadded:: 2.0.1$nbsp;(Plugin)
    def get_date_time_callback_period
      check_validity

      send_request FUNCTION_GET_DATE_TIME_CALLBACK_PERIOD, [], '', 12, 'L'
    end

    # Configures a repeatable alarm. The CALLBACK_ALARM callback is triggered if the
    # current date and time matches the configured alarm.
    # 
    # Setting a parameter to -1 means that it should be disabled and doesn't take part
    # in the match. Setting all parameters to -1 disables the alarm completely.
    # 
    # For example, to make the alarm trigger every day at 7:30 AM it can be
    # configured as (-1, -1, 7, 30, -1, -1, -1). The hour is set to match 7 and the
    # minute is set to match 30. The alarm is triggered if all enabled parameters
    # match.
    # 
    # The interval has a special role. It allows to make the alarm reconfigure itself.
    # This is useful if you need a repeated alarm that cannot be expressed by matching
    # the current date and time. For example, to make the alarm trigger every 23
    # seconds it can be configured as (-1, -1, -1, -1, -1, -1, 23). Internally the
    # Bricklet will take the current date and time, add 23 seconds to it and set the
    # result as its alarm. The first alarm will be triggered 23 seconds after the
    # call. Because the interval is not -1, the Bricklet will do the same again
    # internally, take the current date and time, add 23 seconds to it and set that
    # as its alarm. This results in a repeated alarm that triggers every 23 seconds.
    # 
    # The interval can also be used in combination with the other parameters. For
    # example, configuring the alarm as (-1, -1, 7, 30, -1, -1, 300) results in an
    # alarm that triggers every day at 7:30 AM and is then repeated every 5 minutes.
    # 
    # .. versionadded:: 2.0.1$nbsp;(Plugin)
    def set_alarm(month, day, hour, minute, second, weekday, interval)
      check_validity

      send_request FUNCTION_SET_ALARM, [month, day, hour, minute, second, weekday, interval], 'c c c c c c l', 8, ''
    end

    # Returns the alarm configuration as set by BrickletRealTimeClock#set_alarm.
    # 
    # .. versionadded:: 2.0.1$nbsp;(Plugin)
    def get_alarm
      check_validity

      send_request FUNCTION_GET_ALARM, [], '', 18, 'c c c c c c l'
    end

    # Returns the UID, the UID where the Bricklet is connected to,
    # the position, the hardware and firmware version as well as the
    # device identifier.
    # 
    # The position can be 'a', 'b', 'c', 'd', 'e', 'f', 'g' or 'h' (Bricklet Port).
    # A Bricklet connected to an :ref:`Isolator Bricklet <isolator_bricklet>` is always at
    # position 'z'.
    # 
    # The device identifier numbers can be found :ref:`here <device_identifier>`.
    # |device_identifier_constant|
    def get_identity
      send_request FUNCTION_GET_IDENTITY, [], '', 33, 'Z8 Z8 k C3 C3 S'
    end

    # Registers a callback with ID <tt>id</tt> to the block <tt>block</tt>.
    def register_callback(id, &block)
      callback = block
      @registered_callbacks[id] = callback
    end
  end
end
