# -*- ruby encoding: utf-8 -*-
#############################################################
# This file was automatically generated on 2021-05-06.      #
#                                                           #
# Ruby Bindings Version 2.1.29                              #
#                                                           #
# If you have a bugfix for this file and want to commit it, #
# please fix the bug in the generator. You can find a link  #
# to the generators git repository on tinkerforge.com       #
#############################################################

require_relative './ip_connection'

module Tinkerforge
  # Four 7-segment displays with switchable colon
  class BrickletSegmentDisplay4x7 < Device
    DEVICE_IDENTIFIER = 237 # :nodoc:
    DEVICE_DISPLAY_NAME = 'Segment Display 4x7 Bricklet' # :nodoc:

    # This callback is triggered when the counter (see BrickletSegmentDisplay4x7#start_counter) is
    # finished.
    CALLBACK_COUNTER_FINISHED = 5

    FUNCTION_SET_SEGMENTS = 1 # :nodoc:
    FUNCTION_GET_SEGMENTS = 2 # :nodoc:
    FUNCTION_START_COUNTER = 3 # :nodoc:
    FUNCTION_GET_COUNTER_VALUE = 4 # :nodoc:
    FUNCTION_GET_IDENTITY = 255 # :nodoc:


    # Creates an object with the unique device ID <tt>uid</tt> and adds it to
    # the IP Connection <tt>ipcon</tt>.
    def initialize(uid, ipcon)
      super uid, ipcon, DEVICE_IDENTIFIER, DEVICE_DISPLAY_NAME

      @api_version = [2, 0, 0]

      @response_expected[FUNCTION_SET_SEGMENTS] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_GET_SEGMENTS] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_START_COUNTER] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_GET_COUNTER_VALUE] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_GET_IDENTITY] = RESPONSE_EXPECTED_ALWAYS_TRUE

      @callback_formats[CALLBACK_COUNTER_FINISHED] = [8, '']

      @ipcon.add_device self
    end

    # The 7-segment display can be set with bitmaps. Every bit controls one
    # segment:
    # 
    # .. image:: /Images/Bricklets/bricklet_segment_display_4x7_bit_order.png
    #    :scale: 100 %
    #    :alt: Bit order of one segment
    #    :align: center
    # 
    # For example to set a "5" you would want to activate segments 0, 2, 3, 5 and 6.
    # This is represented by the number 0b01101101 = 0x6d = 109.
    # 
    # The brightness can be set between 0 (dark) and 7 (bright). The colon
    # parameter turns the colon of the display on or off.
    def set_segments(segments, brightness, colon)
      check_validity

      send_request FUNCTION_SET_SEGMENTS, [segments, brightness, colon], 'C4 C ?', 8, ''
    end

    # Returns the segment, brightness and color data as set by
    # BrickletSegmentDisplay4x7#set_segments.
    def get_segments
      check_validity

      send_request FUNCTION_GET_SEGMENTS, [], '', 14, 'C4 C ?'
    end

    # Starts a counter with the *from* value that counts to the *to*
    # value with the each step incremented by *increment*.
    # *length* is the pause between each increment.
    # 
    # Example: If you set *from* to 0, *to* to 100, *increment* to 1 and
    # *length* to 1000, a counter that goes from 0 to 100 with one second
    # pause between each increment will be started.
    # 
    # Using a negative increment allows to count backwards.
    # 
    # You can stop the counter at every time by calling BrickletSegmentDisplay4x7#set_segments.
    def start_counter(value_from, value_to, increment, length)
      check_validity

      send_request FUNCTION_START_COUNTER, [value_from, value_to, increment, length], 's s s L', 8, ''
    end

    # Returns the counter value that is currently shown on the display.
    # 
    # If there is no counter running a 0 will be returned.
    def get_counter_value
      check_validity

      send_request FUNCTION_GET_COUNTER_VALUE, [], '', 10, 'S'
    end

    # Returns the UID, the UID where the Bricklet is connected to,
    # the position, the hardware and firmware version as well as the
    # device identifier.
    # 
    # The position can be 'a', 'b', 'c', 'd', 'e', 'f', 'g' or 'h' (Bricklet Port).
    # A Bricklet connected to an :ref:`Isolator Bricklet <isolator_bricklet>` is always at
    # position 'z'.
    # 
    # The device identifier numbers can be found :ref:`here <device_identifier>`.
    # |device_identifier_constant|
    def get_identity
      send_request FUNCTION_GET_IDENTITY, [], '', 33, 'Z8 Z8 k C3 C3 S'
    end

    # Registers a callback with ID <tt>id</tt> to the block <tt>block</tt>.
    def register_callback(id, &block)
      callback = block
      @registered_callbacks[id] = callback
    end
  end
end
