# -*- ruby encoding: utf-8 -*-
#############################################################
# This file was automatically generated on 2021-05-06.      #
#                                                           #
# Ruby Bindings Version 2.1.29                              #
#                                                           #
# If you have a bugfix for this file and want to commit it, #
# please fix the bug in the generator. You can find a link  #
# to the generators git repository on tinkerforge.com       #
#############################################################

require_relative './ip_connection'

module Tinkerforge
  # Controls AC and DC Solid State Relays
  class BrickletSolidStateRelayV2 < Device
    DEVICE_IDENTIFIER = 296 # :nodoc:
    DEVICE_DISPLAY_NAME = 'Solid State Relay Bricklet 2.0' # :nodoc:

    # This callback is triggered whenever the monoflop timer reaches 0.
    # The parameter is the current state of the relay
    # (the state after the monoflop).
    CALLBACK_MONOFLOP_DONE = 5

    FUNCTION_SET_STATE = 1 # :nodoc:
    FUNCTION_GET_STATE = 2 # :nodoc:
    FUNCTION_SET_MONOFLOP = 3 # :nodoc:
    FUNCTION_GET_MONOFLOP = 4 # :nodoc:
    FUNCTION_GET_SPITFP_ERROR_COUNT = 234 # :nodoc:
    FUNCTION_SET_BOOTLOADER_MODE = 235 # :nodoc:
    FUNCTION_GET_BOOTLOADER_MODE = 236 # :nodoc:
    FUNCTION_SET_WRITE_FIRMWARE_POINTER = 237 # :nodoc:
    FUNCTION_WRITE_FIRMWARE = 238 # :nodoc:
    FUNCTION_SET_STATUS_LED_CONFIG = 239 # :nodoc:
    FUNCTION_GET_STATUS_LED_CONFIG = 240 # :nodoc:
    FUNCTION_GET_CHIP_TEMPERATURE = 242 # :nodoc:
    FUNCTION_RESET = 243 # :nodoc:
    FUNCTION_WRITE_UID = 248 # :nodoc:
    FUNCTION_READ_UID = 249 # :nodoc:
    FUNCTION_GET_IDENTITY = 255 # :nodoc:

    BOOTLOADER_MODE_BOOTLOADER = 0 # :nodoc:
    BOOTLOADER_MODE_FIRMWARE = 1 # :nodoc:
    BOOTLOADER_MODE_BOOTLOADER_WAIT_FOR_REBOOT = 2 # :nodoc:
    BOOTLOADER_MODE_FIRMWARE_WAIT_FOR_REBOOT = 3 # :nodoc:
    BOOTLOADER_MODE_FIRMWARE_WAIT_FOR_ERASE_AND_REBOOT = 4 # :nodoc:
    BOOTLOADER_STATUS_OK = 0 # :nodoc:
    BOOTLOADER_STATUS_INVALID_MODE = 1 # :nodoc:
    BOOTLOADER_STATUS_NO_CHANGE = 2 # :nodoc:
    BOOTLOADER_STATUS_ENTRY_FUNCTION_NOT_PRESENT = 3 # :nodoc:
    BOOTLOADER_STATUS_DEVICE_IDENTIFIER_INCORRECT = 4 # :nodoc:
    BOOTLOADER_STATUS_CRC_MISMATCH = 5 # :nodoc:
    STATUS_LED_CONFIG_OFF = 0 # :nodoc:
    STATUS_LED_CONFIG_ON = 1 # :nodoc:
    STATUS_LED_CONFIG_SHOW_HEARTBEAT = 2 # :nodoc:
    STATUS_LED_CONFIG_SHOW_STATUS = 3 # :nodoc:

    # Creates an object with the unique device ID <tt>uid</tt> and adds it to
    # the IP Connection <tt>ipcon</tt>.
    def initialize(uid, ipcon)
      super uid, ipcon, DEVICE_IDENTIFIER, DEVICE_DISPLAY_NAME

      @api_version = [2, 0, 0]

      @response_expected[FUNCTION_SET_STATE] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_GET_STATE] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_SET_MONOFLOP] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_GET_MONOFLOP] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_GET_SPITFP_ERROR_COUNT] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_SET_BOOTLOADER_MODE] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_GET_BOOTLOADER_MODE] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_SET_WRITE_FIRMWARE_POINTER] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_WRITE_FIRMWARE] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_SET_STATUS_LED_CONFIG] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_GET_STATUS_LED_CONFIG] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_GET_CHIP_TEMPERATURE] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_RESET] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_WRITE_UID] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_READ_UID] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_GET_IDENTITY] = RESPONSE_EXPECTED_ALWAYS_TRUE

      @callback_formats[CALLBACK_MONOFLOP_DONE] = [9, '?']

      @ipcon.add_device self
    end

    # Sets the state of the relays *true* means on and *false* means off.
    # 
    # A running monoflop timer will be aborted if this function is called.
    def set_state(state)
      check_validity

      send_request FUNCTION_SET_STATE, [state], '?', 8, ''
    end

    # Returns the state of the relay, *true* means on and *false* means off.
    def get_state
      check_validity

      send_request FUNCTION_GET_STATE, [], '', 9, '?'
    end

    # The first parameter  is the desired state of the relay (*true* means on
    # and *false* means off). The second parameter indicates the time that
    # the relay should hold the state.
    # 
    # If this function is called with the parameters (true, 1500):
    # The relay will turn on and in 1.5s it will turn off again.
    # 
    # A monoflop can be used as a failsafe mechanism. For example: Lets assume you
    # have a RS485 bus and a Solid State Relay Bricklet connected to one of the slave
    # stacks. You can now call this function every second, with a time parameter
    # of two seconds. The relay will be on all the time. If now the RS485
    # connection is lost, the relay will turn off in at most two seconds.
    def set_monoflop(state, time)
      check_validity

      send_request FUNCTION_SET_MONOFLOP, [state, time], '? L', 8, ''
    end

    # Returns the current state and the time as set by
    # BrickletSolidStateRelayV2#set_monoflop as well as the remaining time until the state flips.
    # 
    # If the timer is not running currently, the remaining time will be returned
    # as 0.
    def get_monoflop
      check_validity

      send_request FUNCTION_GET_MONOFLOP, [], '', 17, '? L L'
    end

    # Returns the error count for the communication between Brick and Bricklet.
    # 
    # The errors are divided into
    # 
    # * ACK checksum errors,
    # * message checksum errors,
    # * framing errors and
    # * overflow errors.
    # 
    # The errors counts are for errors that occur on the Bricklet side. All
    # Bricks have a similar function that returns the errors on the Brick side.
    def get_spitfp_error_count
      check_validity

      send_request FUNCTION_GET_SPITFP_ERROR_COUNT, [], '', 24, 'L L L L'
    end

    # Sets the bootloader mode and returns the status after the requested
    # mode change was instigated.
    # 
    # You can change from bootloader mode to firmware mode and vice versa. A change
    # from bootloader mode to firmware mode will only take place if the entry function,
    # device identifier and CRC are present and correct.
    # 
    # This function is used by Brick Viewer during flashing. It should not be
    # necessary to call it in a normal user program.
    def set_bootloader_mode(mode)
      check_validity

      send_request FUNCTION_SET_BOOTLOADER_MODE, [mode], 'C', 9, 'C'
    end

    # Returns the current bootloader mode, see BrickletSolidStateRelayV2#set_bootloader_mode.
    def get_bootloader_mode
      check_validity

      send_request FUNCTION_GET_BOOTLOADER_MODE, [], '', 9, 'C'
    end

    # Sets the firmware pointer for BrickletSolidStateRelayV2#write_firmware. The pointer has
    # to be increased by chunks of size 64. The data is written to flash
    # every 4 chunks (which equals to one page of size 256).
    # 
    # This function is used by Brick Viewer during flashing. It should not be
    # necessary to call it in a normal user program.
    def set_write_firmware_pointer(pointer)
      check_validity

      send_request FUNCTION_SET_WRITE_FIRMWARE_POINTER, [pointer], 'L', 8, ''
    end

    # Writes 64 Bytes of firmware at the position as written by
    # BrickletSolidStateRelayV2#set_write_firmware_pointer before. The firmware is written
    # to flash every 4 chunks.
    # 
    # You can only write firmware in bootloader mode.
    # 
    # This function is used by Brick Viewer during flashing. It should not be
    # necessary to call it in a normal user program.
    def write_firmware(data)
      check_validity

      send_request FUNCTION_WRITE_FIRMWARE, [data], 'C64', 9, 'C'
    end

    # Sets the status LED configuration. By default the LED shows
    # communication traffic between Brick and Bricklet, it flickers once
    # for every 10 received data packets.
    # 
    # You can also turn the LED permanently on/off or show a heartbeat.
    # 
    # If the Bricklet is in bootloader mode, the LED is will show heartbeat by default.
    def set_status_led_config(config)
      check_validity

      send_request FUNCTION_SET_STATUS_LED_CONFIG, [config], 'C', 8, ''
    end

    # Returns the configuration as set by BrickletSolidStateRelayV2#set_status_led_config
    def get_status_led_config
      check_validity

      send_request FUNCTION_GET_STATUS_LED_CONFIG, [], '', 9, 'C'
    end

    # Returns the temperature as measured inside the microcontroller. The
    # value returned is not the ambient temperature!
    # 
    # The temperature is only proportional to the real temperature and it has bad
    # accuracy. Practically it is only useful as an indicator for
    # temperature changes.
    def get_chip_temperature
      check_validity

      send_request FUNCTION_GET_CHIP_TEMPERATURE, [], '', 10, 's'
    end

    # Calling this function will reset the Bricklet. All configurations
    # will be lost.
    # 
    # After a reset you have to create new device objects,
    # calling functions on the existing ones will result in
    # undefined behavior!
    def reset
      check_validity

      send_request FUNCTION_RESET, [], '', 8, ''
    end

    # Writes a new UID into flash. If you want to set a new UID
    # you have to decode the Base58 encoded UID string into an
    # integer first.
    # 
    # We recommend that you use Brick Viewer to change the UID.
    def write_uid(uid)
      check_validity

      send_request FUNCTION_WRITE_UID, [uid], 'L', 8, ''
    end

    # Returns the current UID as an integer. Encode as
    # Base58 to get the usual string version.
    def read_uid
      check_validity

      send_request FUNCTION_READ_UID, [], '', 12, 'L'
    end

    # Returns the UID, the UID where the Bricklet is connected to,
    # the position, the hardware and firmware version as well as the
    # device identifier.
    # 
    # The position can be 'a', 'b', 'c', 'd', 'e', 'f', 'g' or 'h' (Bricklet Port).
    # A Bricklet connected to an :ref:`Isolator Bricklet <isolator_bricklet>` is always at
    # position 'z'.
    # 
    # The device identifier numbers can be found :ref:`here <device_identifier>`.
    # |device_identifier_constant|
    def get_identity
      send_request FUNCTION_GET_IDENTITY, [], '', 33, 'Z8 Z8 k C3 C3 S'
    end

    # Registers a callback with ID <tt>id</tt> to the block <tt>block</tt>.
    def register_callback(id, &block)
      callback = block
      @registered_callbacks[id] = callback
    end
  end
end
